@@ -32,7 +32,6 @@
 
 /* Shall we multithread? */
 #undef BALSA_USE_THREADS
-#undef USE_PIXBUF
 
 /* LDAP support? */
 #undef ENABLE_LDAP
@@ -52,3 +51,10 @@
 
 /* use pcre instead of `standard' rexexp's? */
 #undef HAVE_PCRE
+
+/* If Gnome-VFS available? */
+#undef HAVE_GNOME_VFS
+
+/* have isblank? */
+
+#undef isblank
