@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2002 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,10 @@
 #include "misc.h"
 #include "files.h"
 
+#if BALSA_MAJOR > 1
+#define PATH_SEP_STR G_DIR_SEPARATOR_S
+#endif
+
 static const gchar *permanent_prefixes[] = {
 /*	BALSA_DATA_PREFIX,
 	BALSA_STD_PREFIX,
@@ -33,6 +37,8 @@ static const gchar *permanent_prefixes[] = {
 	GNOME_STD_PREFIX,
 	GNOME_LIB_PREFIX,*/
     BALSA_COMMON_PREFIXES,
+    "src",
+    ".",
     NULL
 };
 
@@ -44,7 +50,7 @@ static const gchar *permanent_prefixes[] = {
  */
 gchar *
 balsa_file_finder(const gchar * filename, const gchar * splice,
-		  const gchar ** prefixes)
+		  const gchar ** prefixes, gboolean warn)
 {
     gchar *cat;
     int i;
@@ -66,9 +72,10 @@ balsa_file_finder(const gchar * filename, const gchar * splice,
     }
 
     if (prefixes == NULL) {
-	g_warning
-	    ("Cannot find expected file \"%s\" (spliced with \"%s\") with no extra prefixes",
-	     filename, splice);
+	if (warn)
+            g_warning("Cannot find expected file \"%s\" "
+                      "(spliced with \"%s\") with no extra prefixes",
+	              filename, splice);
 	return NULL;
     }
 
@@ -83,9 +90,10 @@ balsa_file_finder(const gchar * filename, const gchar * splice,
 	g_free(cat);
     }
 
-    g_warning
-	("Cannot find expected file \"%s\" (spliced with \"%s\") even with extra prefixes",
-	 filename, splice);
+    if (warn)
+        g_warning("Cannot find expected file \"%s\" "
+                  "(spliced with \"%s\") even with extra prefixes",
+	          filename, splice);
     return NULL;
 }
 
@@ -97,17 +105,26 @@ balsa_file_finder(const gchar * filename, const gchar * splice,
  *   return the complete path to the icon file.
  */
 gchar *
-libbalsa_icon_finder(const char *mime_type, const char *filename)
+libbalsa_icon_finder(const char *mime_type, const char *filename, 
+                     gchar** used_type)
 {
-    const char *content_type, *icon_file;
+    char *content_type;
+    const char *icon_file;
     gchar *icon = NULL;
     
-    content_type = (!mime_type) ? 
-	libbalsa_lookup_mime_type(filename) : 
-	mime_type;
-
+    if(mime_type)
+        content_type = g_strdup(mime_type);
+    else {
+        if(!filename) {
+            if(used_type) *used_type = g_strdup("application/octet-stream");
+            return balsa_pixmap_finder ("balsa/attachment.png");
+        }
+        content_type = libbalsa_lookup_mime_type(filename);
+    }
+    /* FIXME:
+       or icon_file = gnome_desktop_item_find_icon(GVMGI(content_Type)?) 
+    icon_file = gnome_vfs_mime_get_icon(content_type); */
     icon_file = gnome_mime_get_value (content_type, "icon-filename");
-
     if ( icon_file ) 
 	icon = g_strdup (icon_file);
 
@@ -120,7 +137,7 @@ libbalsa_icon_finder(const char *mime_type, const char *filename)
 	if (p_gnome_icon != NULL)
 	    *p_gnome_icon = '-';
 
-	icon = balsa_pixmap_finder (gnome_icon);
+	icon = balsa_pixmap_finder_no_warn (gnome_icon);
 
 	/*
 	 * FIXME: Should use a better icon. Since this one is small
@@ -131,6 +148,7 @@ libbalsa_icon_finder(const char *mime_type, const char *filename)
 	
 	g_free (gnome_icon);
     }
-
+    if(used_type) *used_type = content_type;
+    else g_free(content_type);
     return (icon);
 }
