@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2002 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -91,7 +91,7 @@ libbalsa_scanner_local_dir(GNode *rnode, const gchar * prefix,
 {
     DIR *dpc;
     struct dirent *de;
-    gchar * name;
+    const gchar * name;
     char filename[PATH_MAX];
     struct stat st;
     GtkType mailbox_type;
@@ -139,6 +139,9 @@ libbalsa_scanner_local_dir(GNode *rnode, const gchar * prefix,
  * --------------------------------------------------------------------- */
 void imap_add_folder (char delim, char *folder, int noselect, int noinferiors,
 		      struct browser_state *state, short isparent);
+/* executed with GDK lock OFF.
+ * see HACKING file for proper locking order description.
+ */
 void
 libbalsa_scanner_imap_dir(GNode *rnode, LibBalsaServer * server, 
 			  const gchar* path, gboolean subscribed, 
@@ -150,8 +153,7 @@ libbalsa_scanner_imap_dir(GNode *rnode, LibBalsaServer * server,
     GList* list = NULL, *el;
     struct browser_state state;
     int i;
-
-    printf("imap_dir: reading for %s\n", path);
+    
     init_state (&state);
     state.imap_browse = 1;
     state.rnode = rnode;
@@ -166,47 +168,57 @@ libbalsa_scanner_imap_dir(GNode *rnode, LibBalsaServer * server,
 	   return;
        }
     }
+    /* try getting password, quit on cancel */
+    if (!server->passwd) {
+        gdk_threads_enter();
+        server->passwd = libbalsa_server_get_password(server, NULL);
+        gdk_threads_leave();
+        if(!server->passwd)
+            return;
+    }
 	
-    libbalsa_lock_mutt();
+    gdk_threads_enter(); libbalsa_lock_mutt();
+
+    if (list_inbox)
+        /* force INBOX into the mailbox list
+         * delim doesn't matter, so we'll give it '/'
+         * and we'll mark it as not scanned, in case it
+         * has any subfolders */
+        mailbox_handler(rnode, "INBOX", '/', FALSE);
+
     safe_free((void **)&ImapUser);   ImapUser = safe_strdup(server->user);
     safe_free((void **)&ImapPass);   ImapPass = safe_strdup(server->passwd);
-
     /* subscribed triggers a bug in libmutt, disable it now */
     if(subscribed)
 	set_option(OPTIMAPLSUB);
     else
 	unset_option(OPTIMAPLSUB);
 
-    if (list_inbox)
-        /* force INBOX into the mailbox list
-         * delim doesn't matter, so we'll give it '/' */
-        mailbox_handler(rnode, "INBOX", '/');
 
     state.subfolders = g_list_append(NULL, g_strdup(path));
     state.folder = NULL;
 
     for(i=0; state.subfolders && i<depth; i++) {
+	state.scanned = (i < depth - 1);
 	list = state.subfolders;
 	state.subfolders = NULL;
-	printf("Deph: %i -------------------------------------------\n", i);
 	for(el= g_list_first(list); el; el = g_list_next(el)) {
 	    if(*(char*)el->data)
-		imap_path = g_strdup_printf("imap%s://%s:%i/%s/", 
+		imap_path = g_strdup_printf("imap%s://%s/%s/", 
 #ifdef USE_SSL
 					    server->use_ssl ? "s" : "",
 #else
 					    "",
 #endif
-					    server->host, server->port, 
-					    (char*)el->data);
+					    server->host, (char*)el->data);
 	    else 
-		imap_path = g_strdup_printf("imap%s://%s:%i/", 
+		imap_path = g_strdup_printf("imap%s://%s/", 
 #ifdef USE_SSL
 					    server->use_ssl ? "s" : "",
 #else
 					    "",
 #endif
-					    server->host, server->port);
+					    server->host);
 	    FREE(&state.folder);
 	    imap_browse ((char*)imap_path,  &state);
 	    g_free(imap_path);
@@ -217,11 +229,14 @@ libbalsa_scanner_imap_dir(GNode *rnode, LibBalsaServer * server,
     g_list_foreach((GList*)state.subfolders, (GFunc)g_free, NULL);
     g_list_free((GList*)state.subfolders);
     state_free (&state);
-    libbalsa_unlock_mutt();
-    
+    libbalsa_unlock_mutt(); gdk_threads_leave();
 }
 
-/* this function ovverrides mutt's one. */
+/* this function ovverrides mutt's one.
+ * called with GDK unlocked via sequence:
+ * libbalsa_scanner_imap_dir->libmutt::imap_browse-> imap_add_folder.
+ * See HACKING file for proper locking order description.
+ */
 void imap_add_folder (char delim, char *folder, int noselect,
   int noinferiors, struct browser_state *state, short isparent)
 {
@@ -233,22 +248,26 @@ void imap_add_folder (char delim, char *folder, int noselect,
 	   "noinferiors: %d, isparent: %d\n", delim, folder, noselect,
 	   noinferiors, isparent);  */
     if(isparent) return;
+    libbalsa_unlock_mutt();
     if(!noselect) {
-	printf("ADDING MAILBOX %s\n", folder);
+	libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
+                             "ADDING MAILBOX %s\n", folder);
 	++isMailbox;
     }
     /* this extra check is needed for subscribed folder handling. 
-	   Read RFC when iin doubt. */
+	   Read RFC when in doubt. */
     if(!g_list_find_custom(state->subfolders, folder,
 			   (GCompareFunc)strcmp)) {
-	printf("ADDING FOLDER  %s\n", folder);
+	libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
+                             "ADDING FOLDER  %s\n", folder);
 	    
 	state->subfolders = g_list_append(state->subfolders,
 					  g_strdup(folder));
 	++isFolder;
     }
     if (isMailbox)
-	state->mailbox_handler(state->rnode, folder, delim);
+	state->mailbox_handler(state->rnode, folder, delim, state->scanned);
     else if (isFolder)
-	state->folder_handler(state->rnode, folder, delim);
+	state->folder_handler(state->rnode, folder, delim, state->scanned);
+    libbalsa_lock_mutt();
 }
