@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2002 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,7 +37,6 @@ typedef struct header MuttHeader;
 #include "message.h"
 #include "body.h"
 #include "files.h"
-#include "misc.h"
 #include "mime.h"
 #include "notify.h"
 
@@ -48,6 +47,8 @@ typedef struct header MuttHeader;
 #include "address-book.h"
 #include "address-book-vcard.h"
 #include "address-book-ldif.h"
+#include "address-book-extern.h"
+
 #if ENABLE_LDAP
 #include "address-book-ldap.h"
 #endif
@@ -61,10 +62,7 @@ typedef struct header MuttHeader;
 #include "mailbox_mh.h"
 #include "mailbox_maildir.h"
 
-#ifdef BALSA_SHOW_ALL
-#include "filter.h"
-#endif
-
+#define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
 /*
  * Initialize the library
  */
@@ -72,12 +70,30 @@ void libbalsa_init(LibBalsaInformationFunc information_callback);
 void libbalsa_set_spool(gchar * spool);
 
 void libbalsa_show_message_source(LibBalsaMessage* msg);
+gchar *libbalsa_rot(const gchar * pass);
 
+gchar *libbalsa_guess_email_address(void);
 gchar *libbalsa_guess_mail_spool(void);
 gboolean libbalsa_is_sending_mail(void);
 void libbalsa_wait_for_sending_thread(gint max_seconds);
 
+gchar *libbalsa_guess_pop_server(void);
+gchar *libbalsa_guess_imap_server(void);
+gchar *libbalsa_guess_ldap_server(void);
+
+gchar *libbalsa_guess_imap_inbox(void);
+
+gchar *libbalsa_guess_ldap_base(void);
+gchar *libbalsa_guess_ldap_name(void);
+
+gchar *libbalsa_guess_ldif_file(void);
+
+gboolean libbalsa_ldap_exists(const gchar *server);
+
 void libbalsa_lock_mutt(void);
 void libbalsa_unlock_mutt(void);
 
+void libbalsa_mktemp (char *);
+void libbalsa_assure_balsa_dir(void);
+
 #endif				/* __LIBBALSA_H__ */
