@@ -20,14 +20,17 @@
  * 02111-1307, USA.
  */
 
-#include <gnome.h>
-
-#include "libmutt/mutt.h"
-
 #ifndef __LIBBALSA_PRIVATE_H__
 #define __LIBBALSA_PRIVATE_H__
 
+#include "libmutt/mutt.h"
+
 LibBalsaAddress *libbalsa_address_new_from_libmutt(ADDRESS * caddr);
+/* private interfaces to avoid the libbalsa API */
+/* address.c: */
+gchar *libbalsa_address_to_gchar_p(LibBalsaAddress * address, gint n);
+/* misc.c: */
+gchar *libbalsa_make_string_from_list_p(const GList * the_list);
 
 
 #ifdef BALSA_USE_THREADS
@@ -119,7 +122,7 @@ do {\
 
 #endif
 
-#define CLIENT_CONTEXT(mailbox)          ((CONTEXT*)mailbox->context)
+#define CLIENT_CONTEXT(mailbox)          (mailbox->context)
 #define CLIENT_CONTEXT_OPEN(mailbox)     (CLIENT_CONTEXT (mailbox) != NULL)
 #define CLIENT_CONTEXT_CLOSED(mailbox)   (CLIENT_CONTEXT (mailbox) == NULL)
 #define RETURN_IF_CLIENT_CONTEXT_CLOSED(mailbox)\
