@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2002 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -69,8 +69,10 @@ typedef struct _LibBalsaMailboxClass LibBalsaMailboxClass;
 struct _LibBalsaMailbox {
     GtkObject object;
 
-    gchar *config_prefix;	/* unique string identifying mailbox in the config file */
-    gchar *name;
+    gchar *config_prefix;       /* unique string identifying mailbox */
+                                /* in the config file                */
+    gchar *name;                /* displayed name for a special mailbox; */
+                                /* Isn't it a GUI thing?                 */
     gchar *url; /* Unique resource locator, file://, imap:// etc */
     /* context refers libmutt internal data.           * 
      * AVOID ACCESSING ITS CONTENT SINCE THE STRUCTURE *
@@ -81,6 +83,7 @@ struct _LibBalsaMailbox {
     gboolean lock;
     gboolean is_directory;
     gboolean readonly;
+    gboolean disconnected;
 
     glong messages; /* NOTE: this is used for internal msg counting;
 		     * it is often different from g_list_count(messages) */  
@@ -94,13 +97,19 @@ struct _LibBalsaMailbox {
 
     /* Mailing list contained in this mailbox. Or NULL */
     LibBalsaAddress *mailing_list_address;
+
+    /* Associated filters (struct mailbox_filter) */
+    GSList * filters;
+
+    /* Default identity associated with the mailbox */
+    gchar *identity_name;
 };
 
 struct _LibBalsaMailboxClass {
     GtkObjectClass parent_class;
 
     /* Signals */
-    void (*open_mailbox) (LibBalsaMailbox * mailbox);
+    gboolean (*open_mailbox) (LibBalsaMailbox * mailbox);
     LibBalsaMailboxAppendHandle* (*open_mailbox_append)
 	(LibBalsaMailbox * mailbox);
     void (*close_mailbox) (LibBalsaMailbox * mailbox);
@@ -124,6 +133,8 @@ struct _LibBalsaMailboxClass {
     FILE *(*get_message_stream) (LibBalsaMailbox * mailbox,
 				 LibBalsaMessage * message);
     void (*check) (LibBalsaMailbox * mailbox);
+    GHashTable* (*get_matching)(LibBalsaMailbox * mailbox,
+                                int op, GSList* conditions);
     void (*save_config) (LibBalsaMailbox * mailbox, const gchar * prefix);
     void (*load_config) (LibBalsaMailbox * mailbox, const gchar * prefix);
 };
@@ -137,11 +148,13 @@ LibBalsaMailbox *libbalsa_mailbox_new_from_config(const gchar * prefix);
  * open and close a mailbox 
  */
 /* XXX these need to return a value if they failed */
-void libbalsa_mailbox_open(LibBalsaMailbox * mailbox);
+gboolean libbalsa_mailbox_open(LibBalsaMailbox * mailbox);
+gboolean libbalsa_mailbox_is_valid(LibBalsaMailbox * mailbox);
 LibBalsaMailboxAppendHandle* 
 libbalsa_mailbox_open_append(LibBalsaMailbox * mailbox);
 int libbalsa_mailbox_close_append(LibBalsaMailboxAppendHandle* handle);
 void libbalsa_mailbox_close(LibBalsaMailbox * mailbox);
+void libbalsa_mailbox_link_message(LibBalsaMailbox * mbx, LibBalsaMessage*msg);
 void libbalsa_mailbox_load_messages(LibBalsaMailbox * mailbox);
 
 void libbalsa_mailbox_free_messages(LibBalsaMailbox * mailbox);
@@ -151,17 +164,13 @@ void libbalsa_mailbox_set_unread_messages_flag(LibBalsaMailbox * mailbox,
 
 FILE *libbalsa_mailbox_get_message_stream(LibBalsaMailbox * mailbox,
 					  LibBalsaMessage * message);
-
-/*
- * sorting mailbox
- */
-void libbalsa_mailbox_sort(LibBalsaMailbox * mailbox,
-			   LibBalsaMailboxSort sort);
-
-gint libbalsa_mailbox_sync_backend(LibBalsaMailbox * mailbox);
+gint libbalsa_mailbox_sync_backend(LibBalsaMailbox * mailbox, gboolean delete);
 
 void libbalsa_mailbox_check(LibBalsaMailbox * mailbox);
 
+GHashTable*libbalsa_mailbox_get_matching(LibBalsaMailbox * mailbox,
+                                         int op, GSList* conditions);
+
 void libbalsa_mailbox_save_config(LibBalsaMailbox * mailbox,
 				  const gchar * prefix);
 void libbalsa_mailbox_load_config(LibBalsaMailbox * mailbox,
@@ -172,4 +181,5 @@ void libbalsa_mailbox_load_config(LibBalsaMailbox * mailbox,
 GtkType libbalsa_mailbox_type_from_path(const gchar * filename);
 gboolean libbalsa_mailbox_commit(LibBalsaMailbox* mailbox);
 
+
 #endif				/* __LIBBALSA_MAILBOX_H__ */
