@@ -33,7 +33,13 @@
 #include <auth-client.h>
 #endif
 
-/* #define MESSAGE_COPY_CONTENT 1 */
+/* do copy content, otherwise we get too many races conditions.
+   Most common was to grab subject and then see libmutt delete its data
+   under us (get_subject errors). The solution would be to 
+   lock data harder seem to require a major reengineering.
+*/
+#define MESSAGE_COPY_CONTENT 1
+
 #define LIBBALSA_TYPE_MESSAGE                      (libbalsa_message_get_type())
 #define LIBBALSA_MESSAGE(obj)                      (GTK_CHECK_CAST(obj, LIBBALSA_TYPE_MESSAGE, LibBalsaMessage))
 #define LIBBALSA_MESSAGE_CLASS(klass)              (GTK_CHECK_CLASS_CAST(klass, LIBBALSA_TYPE_MESSAGE, LibBalsaMessageClass))
@@ -89,8 +95,8 @@ struct _LibBalsaMessage {
     GList *cc_list;
     GList *bcc_list;
 
-    /* File Carbon Copy Mailbox */
-    gchar *fcc_mailbox;
+    /* File Carbon Copy Mailbox URL */
+    gchar *fcc_url;
 
     /* replied message ID's */
     GList *references;
@@ -99,6 +105,9 @@ struct _LibBalsaMessage {
     /* replied message ID; from address on date */
     gchar *in_reply_to;
 
+    /* miscellaneous headers */
+    GList *user_headers;
+    
     /* message ID */
     gchar *message_id;
 
@@ -209,12 +218,21 @@ gchar *libbalsa_message_date_to_gchar(LibBalsaMessage * message,
 				      const gchar * date_string);
 gchar *libbalsa_message_size_to_gchar(LibBalsaMessage * message,
                                       gboolean lines);
+gchar *libbalsa_message_title(LibBalsaMessage * message,
+                              const gchar * format);
+gchar **libbalsa_create_hdr_pair(const gchar * name, gchar * value);
 
 const gchar *libbalsa_message_pathname(LibBalsaMessage * message);
 const gchar *libbalsa_message_charset(LibBalsaMessage * message);
+const gchar *libbalsa_message_body_charset(LibBalsaMessageBody * body);
+gboolean libbalsa_message_is_multipart(LibBalsaMessage * message);
 gboolean libbalsa_message_has_attachment(LibBalsaMessage * message);
 
 GList *libbalsa_message_user_hdrs(LibBalsaMessage * message);
+GList *libbalsa_message_find_user_hdr(LibBalsaMessage * message, 
+                                      const gchar * find);
+LibBalsaMessage *libbalsa_message_find_by_message_id(LibBalsaMailbox * mailbox,
+                                                     gchar * msgid);
 
 void libbalsa_message_set_dispnotify(LibBalsaMessage *message, 
 				     LibBalsaAddress *address);
@@ -228,4 +246,5 @@ guint libbalsa_message_get_lines(LibBalsaMessage* msg);
 glong libbalsa_message_get_length(LibBalsaMessage* msg);
 #endif
 glong libbalsa_message_get_no(LibBalsaMessage* msg);
+void libbalsa_message_headers_update(LibBalsaMessage * message);
 #endif				/* __LIBBALSA_MESSAGE_H__ */
