@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2002 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,17 @@
 
 #include "libbalsa.h"
 #include "libbalsa_private.h"
-#include "mailbackend.h"
+#include "misc.h"
+
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-config.h> 
+#include <libgnome/gnome-i18n.h> 
+
+
+/* GTK_CLASS_TYPE for 1.2<->1.3/2.0 GTK+ compatibility */
+#ifndef GTK_CLASS_TYPE
+#define GTK_CLASS_TYPE(x) (GTK_OBJECT_CLASS(x)->type)
+#endif /* GTK_CLASS_TYPE */
 
 static GtkObjectClass *parent_class = NULL;
 static void libbalsa_server_class_init(LibBalsaServerClass * klass);
@@ -36,7 +46,7 @@ static void libbalsa_server_real_set_username(LibBalsaServer * server,
 static void libbalsa_server_real_set_password(LibBalsaServer * server,
 					      const gchar * passwd);
 static void libbalsa_server_real_set_host(LibBalsaServer * server,
-					  const gchar * host, gint port
+					  const gchar * host
 #ifdef USE_SSL
 					  ,gboolean use_ssl
 #endif
@@ -89,26 +99,28 @@ libbalsa_server_class_init(LibBalsaServerClass * klass)
     libbalsa_server_signals[SET_USERNAME] =
 	gtk_signal_new("set-username",
 		       GTK_RUN_FIRST,
-		       object_class->type,
+		       GTK_CLASS_TYPE(object_class),
 		       GTK_SIGNAL_OFFSET(LibBalsaServerClass,
 					 set_username),
 		       gtk_marshal_NONE__STRING, GTK_TYPE_NONE, 1,
 		       GTK_TYPE_STRING);
     libbalsa_server_signals[SET_PASSWORD] =
-	gtk_signal_new("set-password", GTK_RUN_FIRST, object_class->type,
+	gtk_signal_new("set-password", GTK_RUN_FIRST,
+                       GTK_CLASS_TYPE(object_class),
 		       GTK_SIGNAL_OFFSET(LibBalsaServerClass,
 					 set_password),
 		       gtk_marshal_NONE__STRING, GTK_TYPE_NONE, 1,
 		       GTK_TYPE_STRING);
     libbalsa_server_signals[SET_HOST] =
-	gtk_signal_new("set-host", GTK_RUN_FIRST, object_class->type,
+	gtk_signal_new("set-host", GTK_RUN_FIRST,
+                       GTK_CLASS_TYPE(object_class),
 		       GTK_SIGNAL_OFFSET(LibBalsaServerClass, set_host),
 #ifdef USE_SSL
-		       gtk_marshal_NONE__POINTER_INT_INT, GTK_TYPE_NONE, 3,
-		       GTK_TYPE_STRING, GTK_TYPE_INT, GTK_TYPE_BOOL);
+		       gtk_marshal_NONE__POINTER_INT, GTK_TYPE_NONE, 2,
+		       GTK_TYPE_STRING, GTK_TYPE_BOOL);
 #else
-                       gtk_marshal_NONE__POINTER_INT, GTK_TYPE_NONE, 2,
-	               GTK_TYPE_STRING, GTK_TYPE_INT);
+                       gtk_marshal_NONE__POINTER, GTK_TYPE_NONE, 1,
+	               GTK_TYPE_STRING);
 #endif
 
 
@@ -134,9 +146,9 @@ static void
 libbalsa_server_init(LibBalsaServer * server)
 {
     server->host = NULL;
-    server->port = 0;
     server->user = NULL;
     server->passwd = NULL;
+    server->remember_passwd = TRUE;
 #ifdef USE_SSL
     server->use_ssl = FALSE;
 #endif
@@ -192,8 +204,7 @@ libbalsa_server_set_password(LibBalsaServer * server, const gchar * passwd)
 }
 
 void
-libbalsa_server_set_host(LibBalsaServer * server, const gchar * host,
-			 gint port
+libbalsa_server_set_host(LibBalsaServer * server, const gchar * host
 #ifdef USE_SSL
 			 , gboolean use_ssl
 #endif
@@ -203,7 +214,7 @@ libbalsa_server_set_host(LibBalsaServer * server, const gchar * host,
     g_return_if_fail(LIBBALSA_IS_SERVER(server));
 
     gtk_signal_emit(GTK_OBJECT(server), libbalsa_server_signals[SET_HOST],
-		    host, port
+		    host
 #ifdef USE_SSL
 		    , use_ssl
 #endif
@@ -248,8 +259,7 @@ libbalsa_server_real_set_password(LibBalsaServer * server,
 }
 
 static void
-libbalsa_server_real_set_host(LibBalsaServer * server, const gchar * host,
-			      gint port
+libbalsa_server_real_set_host(LibBalsaServer * server, const gchar * host
 #ifdef USE_SSL
 			      , gboolean use_ssl
 #endif
@@ -259,7 +269,6 @@ libbalsa_server_real_set_host(LibBalsaServer * server, const gchar * host,
 
     g_free(server->host);
     server->host = g_strdup(host);
-    server->port = port;
 #ifdef USE_SSL
     server->use_ssl = use_ssl;
 #endif 
@@ -276,48 +285,36 @@ libbalsa_server_real_get_password(LibBalsaServer * server)
 }
 #endif
 
-static gchar *
-rot(gchar * pass)
-{
-    gchar *buff;
-    gint len = 0, i = 0;
-
-    /*PKGW: let's do the assert() BEFORE we coredump... */
-    g_assert(pass != NULL);
-
-    len = strlen(pass);
-    buff = g_strdup(pass);
-
-    for (i = 0; i < len; i++) {
-	if ((buff[i] <= 'M' && buff[i] >= 'A')
-	    || (buff[i] <= 'm' && buff[i] >= 'a'))
-	    buff[i] += 13;
-	else if ((buff[i] <= 'Z' && buff[i] >= 'N')
-		 || (buff[i] <= 'z' && buff[i] >= 'n'))
-	    buff[i] -= 13;
-    }
-    return buff;
-}
 
 /* libbalsa_server_load_config:
    load the server configuration using gnome-config.
-   Try to use sensible defaults.
+   Try to use sensible defaults. 
+   FIXME: Port field is kept here only for compatibility, drop after 1.4.x
+   release.
 */
 void
-libbalsa_server_load_config(LibBalsaServer * server, gint default_port)
+libbalsa_server_load_config(LibBalsaServer * server)
 {
     gboolean d;
     server->host = gnome_config_get_string("Server");
-    server->port = gnome_config_get_int_with_default("Port", &d);
-    if (d)
-	server->port = default_port;
+    if(strrchr(server->host, ':') == NULL) {
+        gint port;
+        port = gnome_config_get_int_with_default("Port", &d);
+        if (!d) {
+            gchar *newhost = g_strdup_printf("%s:%d", server->host, port);
+            g_free(server->host);
+            server->host = newhost;
+        }
+    }       
 #ifdef USE_SSL
     server->use_ssl = gnome_config_get_bool_with_default("SSL", &d);
     if (d)
 	server->use_ssl = FALSE;
 #endif
     server->user = gnome_config_private_get_string("Username");
-    server->passwd = gnome_config_private_get_string("Password");
+    server->remember_passwd = gnome_config_get_bool("RememberPasswd=true");
+    if(server->remember_passwd)
+        server->passwd = gnome_config_private_get_string("Password");
     if(server->passwd && server->passwd[0] == '\0') {
 	g_free(server->passwd);
 	server->passwd = NULL;
@@ -327,8 +324,7 @@ libbalsa_server_load_config(LibBalsaServer * server, gint default_port)
 	server->user = g_strdup(getenv("USER"));
 
     if (server->passwd != NULL) {
-	gchar *buff;
-	buff = rot(server->passwd);
+	gchar *buff = libbalsa_rot(server->passwd);
 	g_free(server->passwd);
 	server->passwd = buff;
     }
@@ -338,16 +334,15 @@ void
 libbalsa_server_save_config(LibBalsaServer * server)
 {
     gnome_config_set_string("Server", server->host);
-    gnome_config_set_int("Port", server->port);
     gnome_config_private_set_string("Username", server->user);
+    gnome_config_set_bool("RememberPasswd", server->remember_passwd);
 
-    if (server->passwd != NULL) {
-	gchar *buff;
-	buff = rot(server->passwd);
+    if (server->remember_passwd && server->passwd != NULL) {
+	gchar *buff = libbalsa_rot(server->passwd);
 	gnome_config_private_set_string("Password", buff);
 	g_free(buff);
     }
 #ifdef USE_SSL
-    gnome_config_set_bool("SSL", server->use_ssl );
+    gnome_config_set_bool("SSL", server->use_ssl);
 #endif
 }
