@@ -48,13 +48,13 @@ struct _LibBalsaServer {
     LibBalsaServerType type;
 
     gchar *host;
-    gint port;
 #ifdef USE_SSL
     gboolean use_ssl;
 #endif
 
     gchar *user;
-    gchar *passwd;
+    gchar *passwd; /* NULL means "ask for it". Empty ("") is a legal one */
+    gboolean remember_passwd;
 };
 
 struct _LibBalsaServerClass {
@@ -63,7 +63,7 @@ struct _LibBalsaServerClass {
     void (*set_username) (LibBalsaServer * server, const gchar * name);
     void (*set_password) (LibBalsaServer * server, const gchar * passwd);
     void (*set_host) (LibBalsaServer * server,
-		      const gchar * host, gint port
+		      const gchar * host
 #ifdef USE_SSL
 		      , gboolean use_ssl
 #endif
@@ -77,8 +77,7 @@ void libbalsa_server_set_username(LibBalsaServer * server,
 				  const gchar * username);
 void libbalsa_server_set_password(LibBalsaServer * server,
 				  const gchar * passwd);
-void libbalsa_server_set_host(LibBalsaServer * server, const gchar * host,
-			      gint port
+void libbalsa_server_set_host(LibBalsaServer * server, const gchar * host
 #ifdef USE_SSL
 			      , gboolean use_ssl
 #endif
@@ -86,7 +85,7 @@ void libbalsa_server_set_host(LibBalsaServer * server, const gchar * host,
 gchar *libbalsa_server_get_password(LibBalsaServer * server,
 				    LibBalsaMailbox * mbox);
 
-void libbalsa_server_load_config(LibBalsaServer * server, gint port);
+void libbalsa_server_load_config(LibBalsaServer * server);
 void libbalsa_server_save_config(LibBalsaServer * server);
 
 #endif				/* __LIBBALSA_SERVER_H__ */
