@@ -1,6 +1,7 @@
 
 /* Enable debugging info */
-#define DEBUG
+#undef DEBUG
+#define NDEBUG
 
 /* Disable the X-Mailer header? */
 #undef NO_XMAILER
@@ -24,12 +25,20 @@
  */
 #undef NFS_ATTRIBUTE_HACK
 
+/* Include code for socket support. Set automatically if you enable pop or
+ * IMAP */
+#undef USE_SOCKET
+
 /* Do you want support for the POP3 protocol? (--enable-pop) */
 #undef USE_POP
 
 /* Do you want support for the IMAP protocol? (--enable-imap) */
 #undef USE_IMAP
 
+/* Do you want to use the Cyrus SASL library for POP/IMAP authentication?
+ * (--with-sasl) */
+#undef USE_SASL
+
 /* Do you want support for IMAP GSSAPI authentication? (--with-gss) */
 #undef USE_GSS
 
@@ -95,3 +104,17 @@
 
 /* Does your system have the fchdir() call? */
 #undef HAVE_FCHDIR
+
+/* BALSA: use pcre instead of `standard' rexexp's? */
+#undef HAVE_PCRE
+
+/* Define if you have iconv(). */
+#define HAVE_ICONV 1
+
+/* Define as 1 if iconv() only converts exactly and we should treat
+ * all return values other than (size_t)(-1) as equivalent. */
+#define ICONV_NONTRANS 0
+
+/* Define if GSS_C_NT_HOSTBASED_SERVICE is defined in the gssapi.h
+   header file.  Only relevant when using GSSAPI.  */
+#undef HAVE_GSS_C_NT_HOSTBASED_SERVICE
