@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 2000-1 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -19,53 +20,62 @@
 #ifndef _IMAP_H
 #define _IMAP_H 1
 
-/* BALSA: we don't use terminal browser #include "browser.h" */
+#include "account.h"
+#include "browser.h"
 #include "mailbox.h"
 
+/* -- data structures -- */
 typedef struct
 {
-  char user[64];
-  char pass[32];
-  char host[128];
-  int port;
-  char type[16];
-  int socktype;
-  char *mbox;
-  int flags;
+  ACCOUNT account;
+  char* mbox;
 } IMAP_MBOX;
 
-
 /* imap.c */
+int imap_access (const char*, int);
 int imap_check_mailbox (CONTEXT *ctx, int *index_hint);
-int imap_create_mailbox (CONTEXT* idata, char* mailbox);
 int imap_close_connection (CONTEXT *ctx);
-int imap_delete_mailbox (CONTEXT* idata, char* mailbox);
+int imap_delete_mailbox (CONTEXT* idata, IMAP_MBOX mx);
 int imap_open_mailbox (CONTEXT *ctx);
 int imap_open_mailbox_append (CONTEXT *ctx);
-int imap_select_mailbox (CONTEXT *ctx, const char* path);
-void imap_set_logout (CONTEXT *ctx);
 int imap_sync_mailbox (CONTEXT *ctx, int expunge, int *index_hint);
-void imap_fastclose_mailbox (CONTEXT *ctx);
+void imap_close_mailbox (CONTEXT *ctx);
 int imap_buffy_check (char *path);
+#ifdef LIBMUTT
+int imap_mailbox_check(char *path, int new,
+		       int imap_check_test(const char *mbox));
+#else
 int imap_mailbox_check (char *path, int new);
-int imap_subscribe (char *path, int subscribe);
+#endif
+int imap_subscribe (const char *path, int subscribe);
 int imap_complete (char* dest, size_t dlen, char* path);
 
 void imap_allow_reopen (CONTEXT *ctx);
 void imap_disallow_reopen (CONTEXT *ctx);
 
 /* browse.c */
-/* BALSA: int imap_init_browse (char *path, struct browser_state *state); */
+int imap_browse (char* path, struct browser_state* state);
+/* LIBMUTT - BALSA: changed prototype for imap_mailbox_create: */
+int imap_mailbox_create (const char* folder, const char* subfolder, int subscribe);
+/* BALSA: prototype for new function imap_mailbox_rename: */
+int imap_mailbox_rename (const char* url, const char* parent, 
+                         const char* subfolder, int subscribe);
+/* BALSA: prototype for new function imap_mailbox_delete: */
+int imap_mailbox_delete (const char* path);
+
 
 /* message.c */
 int imap_append_message (CONTEXT* ctx, MESSAGE* msg);
 int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete);
 int imap_fetch_message (MESSAGE* msg, CONTEXT* ctx, int msgno);
 
+/* socket.c */
+void imap_logout_all (void);
+
 /* util.c */
-int imap_parse_path (const char* path, IMAP_MBOX *mx);
-void imap_qualify_path (char* dest, size_t len, const IMAP_MBOX *mx,
-  const char* path, const char* name);
+int imap_expand_path (char* path, size_t len);
+int imap_parse_path (const char* path, IMAP_MBOX* mx);
+void imap_pretty_mailbox (char* path);
 
 int imap_wait_keepalive (pid_t pid);
 void imap_keepalive (void);
