@@ -40,15 +40,21 @@ struct _AddressBookConfig {
     GtkWidget *name_entry;
     GtkWidget *expand_aliases_button;
 
+    gchar* help_path;
     GtkType create_type;
 
     union {
 	struct {
 	    GtkWidget *path;
 	} vcard;
+	struct {
+		GtkWidget *load;
+		GtkWidget *save;
+	} externq;
 	struct {
 	    GtkWidget *path;
 	} ldif;
+	
 #ifdef ENABLE_LDAP
 	struct {
 	    GtkWidget *host_name;
@@ -62,11 +68,13 @@ struct _AddressBookConfig {
 
 static GtkWidget *create_choice_page(AddressBookConfig * abc);
 static GtkWidget *create_vcard_page(AddressBookConfig * abc);
+static GtkWidget *create_externq_page(AddressBookConfig * abc);
 static GtkWidget *create_ldif_page(AddressBookConfig * abc);
 #ifdef ENABLE_LDAP
 static GtkWidget *create_ldap_page(AddressBookConfig * abc);
 #endif
 
+static void help_button_cb(GtkWidget * button, AddressBookConfig * abc);
 static void cancel_button_cb(GtkWidget * button, AddressBookConfig * abc);
 static void next_button_cb(GtkWidget * button, AddressBookConfig * abc);
 static void add_button_cb(GtkWidget * button, AddressBookConfig * abc);
@@ -74,6 +82,16 @@ static void update_button_cb(GtkWidget * button, AddressBookConfig * abc);
 
 static void set_the_page(GtkWidget * button, AddressBookConfig * abc);
 
+static GtkWidget*
+add_button(const char* butt, GtkWidget* bbox, GtkSignalFunc cb, gpointer abc)
+{
+    GtkWidget* button = gnome_stock_button(butt);
+    gtk_signal_connect(GTK_OBJECT(button), "clicked", cb, abc);
+    gtk_container_add(GTK_CONTAINER(bbox), button);
+    gtk_widget_show(button);
+    return button;
+}
+
 /*
  * Create and run the address book configuration window.
  *
@@ -105,7 +123,8 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book)
     gnome_dialog_set_parent(GNOME_DIALOG(abc->window),
 			    GTK_WINDOW(balsa_app.main_window));
     gnome_dialog_close_hides(GNOME_DIALOG(abc->window), TRUE);
-    gtk_window_set_wmclass(GTK_WINDOW(abc->window), "address_book_config_dialog", "Balsa");
+    gtk_window_set_wmclass(GTK_WINDOW(abc->window), 
+			   "address_book_config_dialog", "Balsa");
 
     abc->notebook = gtk_notebook_new();
     gtk_container_set_border_width(GTK_CONTAINER(abc->window), 5);
@@ -123,16 +142,9 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book)
 				  BALSA_BUTTON_HEIGHT / 2);
 
     if (address_book == NULL) {
-	button = gnome_stock_button(GNOME_STOCK_BUTTON_NEXT);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			   GTK_SIGNAL_FUNC(next_button_cb),
-			   (gpointer) abc);
-	gtk_container_add(GTK_CONTAINER(bbox), button);
-	gtk_widget_show(button);
-	abc->continue_button = button;
-
+	abc->continue_button = 
+	    add_button(GNOME_STOCK_BUTTON_NEXT, bbox, next_button_cb, abc);
 	page = create_choice_page(abc);
-
     } else {
 	GtkWidget *pixmap;
 	pixmap = gnome_stock_pixmap_widget(NULL, GNOME_STOCK_PIXMAP_SAVE);
@@ -146,7 +158,9 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book)
 
 	if (LIBBALSA_IS_ADDRESS_BOOK_VCARD(address_book)) {
 	    page = create_vcard_page(abc);
-	} else if (LIBBALSA_IS_ADDRESS_BOOK_LDIF(address_book)) {
+	}else if (LIBBALSA_IS_ADDRESS_BOOK_EXTERN(address_book)) {
+		page = create_externq_page(abc);
+	}else if (LIBBALSA_IS_ADDRESS_BOOK_LDIF(address_book)) {
 	    page = create_ldif_page(abc);
 #ifdef ENABLE_LDAP
 	} else if (LIBBALSA_IS_ADDRESS_BOOK_LDAP(address_book)) {
@@ -163,11 +177,8 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book)
     num = gtk_notebook_page_num(GTK_NOTEBOOK(abc->notebook), page);
     gtk_notebook_set_page(GTK_NOTEBOOK(abc->notebook), num);
 
-    button = gnome_stock_button(GNOME_STOCK_BUTTON_CANCEL);
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		       GTK_SIGNAL_FUNC(cancel_button_cb), (gpointer) abc);
-    gtk_widget_show(button);
-    gtk_container_add(GTK_CONTAINER(bbox), button);
+    add_button(GNOME_STOCK_BUTTON_CANCEL, bbox, cancel_button_cb, abc);
+    add_button(GNOME_STOCK_BUTTON_HELP,   bbox, help_button_cb,   abc);
 
     gtk_widget_show_all(abc->notebook);
     if (address_book)
@@ -186,10 +197,22 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book)
 	if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_VCARD) {
 	    gchar *path =
 		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY
-					       (abc->ab_specific.vcard.path), FALSE);
+					       (abc->ab_specific.vcard.path),
+					       FALSE);
 	    if (path != NULL) 
 		address_book = libbalsa_address_book_vcard_new(name, path);
             g_free(path);
+ 	} else if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN) {
+	    gchar *load =
+ 		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY
+ 					       (abc->ab_specific.externq.load),
+					       FALSE);
+		gchar *save = 
+ 		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY
+ 					       (abc->ab_specific.externq.save),
+					       FALSE);
+ 	    if (load != NULL && save != NULL) 
+ 		address_book = libbalsa_address_book_externq_new(name, load, save);
 	} else if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF) {
 	    gchar *path =
 		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY
@@ -230,6 +253,26 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book)
 		g_free(vcard->path);
 		vcard->path = path;
 	    }
+ 	} else if (LIBBALSA_IS_ADDRESS_BOOK_EXTERN(address_book)) {
+ 	    LibBalsaAddressBookExtern *externq;
+ 	    gchar *load =
+ 		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY
+ 					       (abc->ab_specific.externq.load),
+					       FALSE);
+ 	    gchar *save =
+ 		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY
+ 					       (abc->ab_specific.externq.save),
+					       FALSE);
+ 
+ 	    externq = LIBBALSA_ADDRESS_BOOK_EXTERN(address_book);
+ 	    if (load) {
+ 		g_free(externq->load);
+ 		externq->load = load;;
+	    }
+ 	    if (save) {
+ 		g_free(externq->save);
+ 		externq->save = save;
+	    }
 	} else if (LIBBALSA_IS_ADDRESS_BOOK_LDIF(address_book)) {
 	    LibBalsaAddressBookLdif *ldif;
 	    gchar *path =
@@ -278,6 +321,8 @@ create_choice_page(AddressBookConfig * abc)
     GtkWidget *vbox;
     GtkWidget *radio_button;
 
+    abc->help_path = "ab-conf.html#CHOICE";
+
     vbox = gtk_vbox_new(FALSE, 0);
     gtk_widget_show(vbox);
 
@@ -290,8 +335,7 @@ create_choice_page(AddressBookConfig * abc)
     /* ... vCard Address Book */
     radio_button =
 	gtk_radio_button_new_with_label(NULL,
-					_
-					("VCard Address Book (GnomeCard)"));
+					_("VCard Address Book (GnomeCard)"));
     gtk_box_pack_start(GTK_BOX(vbox), radio_button, FALSE, FALSE, 0);
     gtk_signal_connect(GTK_OBJECT(radio_button), "clicked",
 		       GTK_SIGNAL_FUNC(set_the_page), (gpointer) abc);
@@ -302,6 +346,19 @@ create_choice_page(AddressBookConfig * abc)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_button), TRUE);
     abc->create_type = LIBBALSA_TYPE_ADDRESS_BOOK_VCARD;
 
+	/* ... External query Address Book */
+    radio_button = gtk_radio_button_new_with_label
+	(gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button)),
+	 _("External query (a program)"));
+    gtk_box_pack_start(GTK_BOX(vbox), radio_button, FALSE, FALSE, 0);
+    gtk_signal_connect(GTK_OBJECT(radio_button), "clicked",
+		       GTK_SIGNAL_FUNC(set_the_page), (gpointer) abc);
+    gtk_object_set_user_data(GTK_OBJECT(radio_button),
+			     GINT_TO_POINTER
+			     (LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN));
+    gtk_widget_show(radio_button);
+
+
     /* ... LDIF Address Book */
     radio_button = gtk_radio_button_new_with_label
 	(gtk_radio_button_group(GTK_RADIO_BUTTON(radio_button)),
@@ -350,6 +407,7 @@ create_vcard_page(AddressBookConfig * abc)
     LibBalsaAddressBookVcard* ab;
     ab = (LibBalsaAddressBookVcard*)abc->address_book; /* may be NULL */
 
+    abc->help_path = "ab-conf.html#VCARD";
     table = gtk_table_new(2, 3, FALSE);
 
     /* mailbox name */
@@ -366,7 +424,7 @@ create_vcard_page(AddressBookConfig * abc)
 
     abc->ab_specific.vcard.path =
 	gnome_file_entry_new("VCARD ADDRESS BOOK PATH",
-			     _("Select path for address book"));
+			     _("Select path for VCARD address book"));
     gtk_table_attach(GTK_TABLE(table), abc->ab_specific.vcard.path, 1, 2,
 		     1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
 
@@ -385,6 +443,68 @@ create_vcard_page(AddressBookConfig * abc)
 
 }
 
+static GtkWidget *
+create_externq_page(AddressBookConfig * abc)
+{
+    GtkWidget *table;
+    GnomeDialog* mcw = GNOME_DIALOG(abc->window);
+    GtkWidget *label_load;
+	GtkWidget *label_save;
+    guint keyval;
+    LibBalsaAddressBookExtern* ab;
+    ab = (LibBalsaAddressBookExtern*)abc->address_book; /* may be NULL */
+    abc->help_path = "ab-conf.html#EXTERN";
+    table = gtk_table_new(3, 3, FALSE);
+
+    /* mailbox name */
+
+    create_label(_("_Address Book Name"), table, 0, &keyval);
+    abc->name_entry = create_entry(mcw, table, NULL, NULL, 0, 
+				   ab ? abc->address_book->name : NULL, 
+				   keyval);
+
+    label_load = gtk_label_new(_("Load program location"));
+    gtk_misc_set_alignment(GTK_MISC(label_load), 1.0, 0.5);
+    gtk_table_attach(GTK_TABLE(table), label_load, 0, 1, 1, 2,
+		     GTK_FILL, GTK_FILL, 10, 10);
+
+    label_save = gtk_label_new(_("Save program location"));
+    gtk_misc_set_alignment(GTK_MISC(label_save), 1.0, 0.5);
+    gtk_table_attach(GTK_TABLE(table), label_save, 0, 1, 2, 3,
+		     GTK_FILL, GTK_FILL, 10, 10);
+
+    abc->ab_specific.externq.load =
+	gnome_file_entry_new("EXTERN ADDRESS BOOK LOAD PATH",
+			     _("Select load program for address book"));
+    gtk_table_attach(GTK_TABLE(table), abc->ab_specific.externq.load, 1, 2,
+		     1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
+
+    abc->ab_specific.externq.save =
+	gnome_file_entry_new("EXTERN ADDRESS BOOK SAVE PATH",
+			     _("Select save program for address book"));
+    gtk_table_attach(GTK_TABLE(table), abc->ab_specific.externq.save, 1, 2,
+		     2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
+    
+	abc->expand_aliases_button =
+	create_check(mcw, _("_Expand aliases as you type"), table, 3,
+		     ab ? abc->address_book->expand_aliases : TRUE);
+
+    if (ab) {
+	GtkWidget *entry;
+	entry = GTK_WIDGET(gnome_file_entry_gtk_entry
+			   (GNOME_FILE_ENTRY(abc->ab_specific.externq.load)));
+	gtk_entry_set_text(GTK_ENTRY(entry), ab->load);
+
+	entry = GTK_WIDGET(gnome_file_entry_gtk_entry
+			   (GNOME_FILE_ENTRY(abc->ab_specific.externq.save)));
+	gtk_entry_set_text(GTK_ENTRY(entry), ab->save);
+    }
+    gtk_widget_show_all(table);
+    return table;
+
+}
+
+
 static GtkWidget *
 create_ldif_page(AddressBookConfig * abc)
 {
@@ -393,6 +513,7 @@ create_ldif_page(AddressBookConfig * abc)
     int keyval;
     GnomeDialog* mcw = GNOME_DIALOG(abc->window);
 
+    abc->help_path = "ab-conf.html#LDIF";
     table = gtk_table_new(2, 3, FALSE);
 
     /* mailbox name */
@@ -410,7 +531,7 @@ create_ldif_page(AddressBookConfig * abc)
 
     abc->ab_specific.ldif.path =
 	gnome_file_entry_new("LDIF ADDRESS BOOK PATH",
-			     _("Select path for address book"));
+			     _("Select path for LDIF address book"));
     gtk_table_attach(GTK_TABLE(table), abc->ab_specific.ldif.path, 1, 2,
 		     1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);
 
@@ -447,31 +568,43 @@ create_ldap_page(AddressBookConfig * abc)
     LibBalsaAddressBookLdap* ab;
     GnomeDialog* mcw = GNOME_DIALOG(abc->window);
     guint keyval;
+    gchar *host = libbalsa_guess_ldap_server();
+    gchar *base = libbalsa_guess_ldap_base();
+    gchar *name = libbalsa_guess_ldap_name();
 
     ab = (LibBalsaAddressBookLdap*)abc->address_book; /* may be NULL */
 
+    abc->help_path = "ab-conf.html#LDAP";
     /* mailbox name */
 
     create_label(_("_Address Book Name"), table, 0, &keyval);
     abc->name_entry = create_entry(mcw, table, NULL, NULL, 0, 
-				   ab ? abc->address_book->name : NULL, 
+				   ab ? abc->address_book->name : name, 
 				   keyval);
 
     create_label(_("_Host Name"), table, 1, &keyval);
     abc->ab_specific.ldap.host_name = 
 	create_entry(mcw, table, NULL, NULL, 1, 
-		     ab ? ab->host : NULL, keyval);
+		     ab ? ab->host : host, keyval);
 
     create_label(_("_Base Domain Name"), table, 2, &keyval);
     abc->ab_specific.ldap.base_dn = 
 	create_entry(mcw, table, NULL, NULL, 2, 
-		     ab ? ab->base_dn : NULL, keyval);
+		     ab ? ab->base_dn : base, keyval);
 
     abc->expand_aliases_button =
 	create_check(mcw, _("_Expand aliases as you type"), table, 3,
 		     ab ? abc->address_book->expand_aliases : TRUE);
 
     gtk_widget_show(table);
+
+    if(base)
+	g_free(base);
+    if(name)
+	g_free(name);
+    if(host)
+	g_free(host);
+
     return table;
 }
 #endif
@@ -489,6 +622,15 @@ set_the_page(GtkWidget * button, AddressBookConfig * abc)
     }
 }
 
+static void
+help_button_cb(GtkWidget * button, AddressBookConfig * abc)
+{
+    static GnomeHelpMenuEntry help_entry = { NULL, NULL };
+    help_entry.name = gnome_app_id;
+    help_entry.path = abc->help_path;
+    gnome_help_display(NULL, &help_entry);
+}
+
 static void
 cancel_button_cb(GtkWidget * button, AddressBookConfig * abc)
 {
@@ -518,6 +660,8 @@ next_button_cb(GtkWidget * button, AddressBookConfig * abc)
 
     if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_VCARD)
 	page = create_vcard_page(abc);
+	else if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_EXTERN)
+	page = create_externq_page(abc);
     else if (abc->create_type == LIBBALSA_TYPE_ADDRESS_BOOK_LDIF)
 	page = create_ldif_page(abc);
 #ifdef ENABLE_LDAP
