@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2002 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -78,34 +78,25 @@ extern "C" {
  * contents is loaded into the index */
     gboolean balsa_index_load_mailbox_node(BalsaIndex * bindex,
                                            BalsaMailboxNode * mbnode);
-    void balsa_index_refresh(BalsaIndex * bindex);
     void balsa_index_update_tree(BalsaIndex *bindex, gboolean expand);
     void balsa_index_set_threading_type(BalsaIndex * bindex, int thtype);
-    void balsa_index_set_sort_order(BalsaIndex * bindex, int column, 
-				    GtkSortType order);
-    void balsa_index_set_first_new_message(BalsaIndex * bindex);
-
-/* adds a new message */
-    void balsa_index_add(BalsaIndex * bindex, LibBalsaMessage * message);
-
-    void balsa_index_update_flag(BalsaIndex * bindex,
-				 LibBalsaMessage * message);
 
     void balsa_index_redraw_current(BalsaIndex *);
 
+/* move or copy a list of messages */
+    void balsa_index_transfer(GList * messages,
+                              LibBalsaMailbox * from_mailbox,
+                              LibBalsaMailbox * to_mailbox,
+                              BalsaIndex * from_bindex, gboolean copy);
+
 /* select up/down the index */
-    void balsa_index_select_next_threaded(BalsaIndex * bindex);
     void balsa_index_select_next(BalsaIndex *);
     void balsa_index_select_next_unread(BalsaIndex * bindex);
     void balsa_index_select_next_flagged(BalsaIndex * bindex);
     void balsa_index_select_previous(BalsaIndex *);
     void balsa_index_select_row(BalsaIndex * bindex, gint row);
 
-/* retrieve the selection */
-    void balsa_index_get_selected_rows(BalsaIndex * bindex,
-				       GtkCTreeNode *** rows,
-				       guint * nb_rows);
-
+    void balsa_index_find(BalsaIndex * bindex,gint op,GSList * conditions,gboolean previous);
 
 /* balsa index page stuff */
     void balsa_message_reply(GtkWidget * widget, gpointer user_data);
@@ -129,14 +120,20 @@ extern "C" {
     gint balsa_find_notebook_page_num(LibBalsaMailbox * mailbox);
     void balsa_index_update_message(BalsaIndex * index);
 
-    /* Threading Stuff */
-    void balsa_index_threading(BalsaIndex* bindex);
-
     /* Updating index columns when preferences change */
     void balsa_index_refresh_date (GtkNotebook *, GtkNotebookPage *,
 				   gint, gpointer);
     void balsa_index_refresh_size (GtkNotebook *, GtkNotebookPage *,
 				   gint, gpointer);
+
+    /* Change the display of all indexes when balsa_app.hide_deleted is
+     * changed */
+    void balsa_index_hide_deleted(gboolean hide);
+    void balsa_index_sync_backend(LibBalsaMailbox * mailbox);
+
+    /* Threading Stuff, implementation is in balsa-index-threading.c */
+    void balsa_index_threading(BalsaIndex* bindex);
+
 #ifdef __cplusplus
 }
 #endif				/* __cplusplus */
