@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
-/* vim:set ts=4 sw=4 ai et: */
+/* vim:set ts=4 sw=4 ai: */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2002 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,12 +34,12 @@
 #include "mime.h"
 #include "misc.h"
 
+/*
 #include <libmutt/mutt.h>
 #include <libmutt/mime.h>
+*/
 
-#ifdef USE_PIXBUF
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#endif
 
 #ifdef HAVE_GTKHTML
 #include <gtkhtml/gtkhtml.h>
@@ -55,7 +55,9 @@
 #include "quote-color.h"
 #include "sendmsg-window.h"
 
-#define ELEMENTS(x) (sizeof (x) / sizeof (x[0]))
+#ifdef HAVE_GNOME_VFS
+# include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#endif
 
 enum {
     SELECT_PART,
@@ -161,6 +163,9 @@ static void part_info_init_unknown(BalsaMessage * bm,
 static void part_info_init_html(BalsaMessage * bm, BalsaPartInfo * info,
 				gchar * ptr, size_t len);
 #endif
+#ifdef HAVE_GNOME_VFS
+static GtkWidget* part_info_mime_button_vfs (BalsaPartInfo* info, const gchar* content_type);
+#endif
 static GtkWidget* part_info_mime_button (BalsaPartInfo* info, const gchar* content_type, const gchar* key);
 static void part_context_menu_call_url(GtkWidget * menu_item, BalsaPartInfo * info);
 static void part_context_menu_mail(GtkWidget * menu_item, BalsaPartInfo * info);
@@ -238,6 +243,7 @@ balsa_message_init(BalsaMessage * bm)
 {
     bm->table = gtk_table_new(3, 1, FALSE);
     gtk_container_add(GTK_CONTAINER(bm), bm->table);
+
     gtk_widget_show(bm->table);
 
     bm->content = gtk_vbox_new(FALSE, 0);
@@ -465,6 +471,7 @@ gboolean
 balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
 {
     gboolean had_focus;
+    gboolean is_new;
 
     g_return_val_if_fail(bm != NULL, FALSE);
 
@@ -492,26 +499,28 @@ balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
 	return TRUE;
     }
 
-    /*
-     * At this point we check if (a) a message has the unread flag set and
-     * (b) a Disposition-Notification-To header line is present
-     */
-    if (message->flags & LIBBALSA_MESSAGE_FLAG_NEW && message->dispnotify_to)
-	handle_mdn_request (message);
-
     bm->message = message;
 
     gtk_signal_connect(GTK_OBJECT(message), "destroy",
 		       GTK_SIGNAL_FUNC(message_destroyed_cb),
 		       (gpointer) bm);
 
-    display_headers(bm);
-
+    is_new = message->flags & LIBBALSA_MESSAGE_FLAG_NEW;
     if(!libbalsa_message_body_ref(bm->message)) 
 	return FALSE;
 
+    display_headers(bm);
     display_content(bm);
 
+    /*
+     * At this point we check if (a) a message was new (its not new
+     * any longer) and (b) a Disposition-Notification-To header line is
+     * present.
+     *
+     */
+    if (is_new && message->dispnotify_to)
+	handle_mdn_request (message);
+
     /*
      * FIXME: This is a workaround for what may or may not be a libmutt bug.
      *
@@ -522,8 +531,10 @@ balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
     if (bm->part_count == 0) {
 	gtk_widget_hide(bm->part_list);
 
-	/* This is really annoying if you are browsing, since you keep getting a dialog... */
-	/* balsa_information(LIBBALSA_INFORMATION_WARNING, _("Message contains no parts!")); */
+	/* This is really annoying if you are browsing, since you keep
+           getting a dialog... */
+	/* balsa_information(LIBBALSA_INFORMATION_WARNING, _("Message
+           contains no parts!")); */
 	return TRUE;
     }
 
@@ -603,17 +614,35 @@ add_header_gchar(BalsaMessage * bm, const gchar *header, const gchar *label,
     gchar cr[] = "\n";
     gchar *line_start, *line_end;
     gchar *wrapped_value;
+    const gchar *msgcharset;
 
     if (!(bm->shown_headers == HEADERS_ALL || libbalsa_find_word(header, balsa_app.selected_headers))) 
 	return;
 
+    /* always display the label in the predefined font */
     if (strcmp(header, "subject") != 0)
 	fnt = gdk_font_load(balsa_app.message_font);
     else
 	fnt = gdk_font_load(balsa_app.subject_font);
 
     gtk_text_insert(GTK_TEXT(bm->header_text), fnt, NULL, NULL, label, -1);
-
+    gdk_font_unref(fnt);
+    
+    /* select the font for the value according to the msg charset */
+    if ((msgcharset = libbalsa_message_charset(bm->message))) {
+	if (strcmp(header, "subject") != 0)
+	    fnt = balsa_get_font_by_charset(balsa_app.message_font, 
+					    msgcharset);
+	else
+	    fnt = balsa_get_font_by_charset(balsa_app.subject_font,
+					    msgcharset);
+    } else {
+	if (strcmp(header, "subject") != 0)
+	    fnt = gdk_font_load(balsa_app.message_font);
+	else
+	    fnt = gdk_font_load(balsa_app.subject_font);
+    }
+	
     if (value && *value != '\0') {
 	if (strlen(label) < 15)
 	    gtk_text_insert(GTK_TEXT(bm->header_text), fnt, NULL, NULL,
@@ -650,6 +679,7 @@ add_header_gchar(BalsaMessage * bm, const gchar *header, const gchar *label,
 	gtk_text_insert(GTK_TEXT(bm->header_text), fnt, NULL, NULL, cr,
 			-1);
     }
+    gdk_font_unref(fnt);
 }
 
 static void
@@ -707,8 +737,8 @@ display_headers(BalsaMessage * bm)
     add_header_glist(bm, "cc", _("Cc:"), message->cc_list);
     add_header_glist(bm, "bcc", _("Bcc:"), message->bcc_list);
 
-    if (message->fcc_mailbox)
-	add_header_gchar(bm, "fcc", _("Fcc:"), message->fcc_mailbox);
+    if (message->fcc_url)
+	add_header_gchar(bm, "fcc", _("Fcc:"), message->fcc_url);
 
     if (message->dispnotify_to) {
 	gchar *mdn_to = libbalsa_address_to_gchar(message->dispnotify_to, 0);
@@ -766,11 +796,7 @@ part_info_init_application(BalsaMessage * bm, BalsaPartInfo * info)
 static void
 part_info_init_image(BalsaMessage * bm, BalsaPartInfo * info)
 {
-#ifndef USE_PIXBUF
-    GdkImlibImage *im;
-#else
     GdkPixbuf *pb;
-#endif
 
     GdkPixmap *pixmap = NULL;
     GdkBitmap *mask = NULL;
@@ -779,19 +805,6 @@ part_info_init_image(BalsaMessage * bm, BalsaPartInfo * info)
 
     libbalsa_message_body_save_temporary(info->body, NULL);
 
-#ifndef USE_PIXBUF
-    if (!(im = gdk_imlib_load_image(info->body->temp_filename))) {
-	g_print(_("Could not load image. It is most likely corrupted."));
-	return;
-    }
-
-    if (!gdk_imlib_render(im, im->rgb_width, im->rgb_height))
-	g_print(_("Couldn't render image\n"));
-
-    pixmap = gdk_imlib_copy_image(im);
-    mask = gdk_imlib_copy_mask(im);
-    gdk_imlib_destroy_image(im);
-#else
     if( (pb = gdk_pixbuf_new_from_file(info->body->temp_filename)) ) {
 	gdk_pixbuf_render_pixmap_and_mask(pb, &pixmap, &mask, 0);
 	gdk_pixbuf_unref(pb);
@@ -800,7 +813,6 @@ part_info_init_image(BalsaMessage * bm, BalsaPartInfo * info)
 	    (_("balsa/pixbuf: Could not load image. It is likely corrupted."));
 	return;
     }
-#endif
 
     if (pixmap) {
 	image = gtk_pixmap_new(pixmap, mask);
@@ -992,8 +1004,11 @@ part_info_init_message_extbody_mail(BalsaMessage * bm, BalsaPartInfo * info)
 static void
 part_info_init_message(BalsaMessage * bm, BalsaPartInfo * info)
 {
-    if (info->body && info->body->mutt_body && info->body->mutt_body->subtype &&
-	!g_strcasecmp("external-body", info->body->mutt_body->subtype)) {
+    gchar* body_type;
+    g_return_if_fail(info->body);
+
+    body_type = libbalsa_message_body_get_content_type(info->body);
+    if (!g_strcasecmp("message/external-body", body_type)) {
 	gchar *access_type;
 	rfc_extbody_id *extbody_type = rfc_extbodys;
 
@@ -1027,6 +1042,7 @@ part_info_init_message(BalsaMessage * bm, BalsaPartInfo * info)
 	g_print("TODO: part_info_init_message\n");
 	part_info_init_unknown(bm, info);
     }
+    g_free(body_type);
 }
 
 static void
@@ -1035,7 +1051,7 @@ part_info_init_unknown(BalsaMessage * bm, BalsaPartInfo * info)
     GtkWidget *vbox;
     GtkWidget *button;
     gchar *msg;
-    const gchar *cmd;
+    const gchar *cmd, *content_desc;
     gchar *content_type;
     
 
@@ -1043,6 +1059,11 @@ part_info_init_unknown(BalsaMessage * bm, BalsaPartInfo * info)
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
 
     content_type = libbalsa_message_body_get_content_type(info->body);
+#ifdef HAVE_GNOME_VFS
+    if((button=part_info_mime_button_vfs(info, content_type))) {
+	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 2);
+    } else
+#endif
     if ((cmd = gnome_mime_get_value(content_type, "view")) != NULL) {
         button = part_info_mime_button (info, content_type, "view");
 	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 2);
@@ -1055,7 +1076,13 @@ part_info_init_unknown(BalsaMessage * bm, BalsaPartInfo * info)
 			   FALSE, FALSE, 1);
     }
 
+#ifdef HAVE_GNOME_VFS
+    if((content_desc=gnome_vfs_mime_get_description(content_type)))
+	msg = g_strdup_printf(_("Type: %s (%s)"), content_desc, content_type);
+    else
+#endif
     msg = g_strdup_printf(_("Content Type: %s"), content_type);
+
     gtk_box_pack_start(GTK_BOX(vbox), gtk_label_new(msg), FALSE, FALSE, 1);
     g_free(msg);
 
@@ -1105,6 +1132,35 @@ part_info_mime_button (BalsaPartInfo* info, const gchar* content_type,
 }
 
 
+#ifdef HAVE_GNOME_VFS
+static GtkWidget*
+part_info_mime_button_vfs (BalsaPartInfo* info, const gchar* content_type)
+{
+    GtkWidget* button=NULL;
+    gchar* msg;
+    const gchar* cmd;
+    GnomeVFSMimeApplication *app=
+	gnome_vfs_mime_get_default_application(content_type);
+
+    if(app) {
+	cmd = app->command;
+	msg = g_strdup_printf(_("View part with %s"), app->name);
+	button = gtk_button_new_with_label(msg);
+	gtk_object_set_data (GTK_OBJECT (button), "mime_action", 
+			     (gpointer) g_strdup(app->id)); /* *** */
+	g_free(msg);
+
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   (GtkSignalFunc) part_context_menu_vfs_cb,
+			   (gpointer) info);
+
+	gnome_vfs_mime_application_free(app);
+	
+    }
+    return button;
+}
+#endif
+
 static void
 display_multipart(BalsaMessage * bm, LibBalsaMessageBody * body)
 {
@@ -1162,6 +1218,8 @@ static struct {
     {"euc-kr", "ksc5601.1987-0", TRUE}, 
     {"koi-8-r", "koi8-r", FALSE}, 
     {"koi-8-u", "koi8-u", FALSE}, 
+    {"koi8-r",  "koi8-r", FALSE}, 
+    {"koi8-u",  "koi8-u", FALSE}, 
     {"us-ascii", "iso8859-1", FALSE}
 };
 
@@ -1179,7 +1237,7 @@ get_font_name(const gchar * base, const gchar * charset,
     int dash_cnt = 0, len, i;
 
     g_return_val_if_fail(base != NULL, NULL);
-    g_return_val_if_fail(charset >= 0, NULL);
+    g_return_val_if_fail(charset != NULL, NULL);
 
     for (i = ELEMENTS(charset2font) - 1; i >= 0; i--)
 	if (g_strcasecmp(charset, charset2font[i].charset) == 0) {
@@ -1203,7 +1261,7 @@ get_font_name(const gchar * base, const gchar * charset,
 
     /* defense against a patologically short base font wildcard implemented
      * in the chunk below
-     * extra space for dwo dashes and '\0' */
+     * extra space for two dashes and '\0' */
     len = ptr - base;
     /* if(dash_cnt>12) len--; */
     if (len < 1)
@@ -1216,10 +1274,8 @@ get_font_name(const gchar * base, const gchar * charset,
 
     if (len > 1)
 	strncpy(res, base, len);
-    else {
-	strncpy(res, "*", 1);
-	len = 1;
-    }
+    else
+	*res='*';
 
     res[len] = '-';
     strcpy(res + len + 1, postfix);
@@ -1674,6 +1730,7 @@ check_over_url(GtkWidget *widget, GdkEvent *event, gpointer data)
     gint x, y;
     GdkModifierType mask;
     static gboolean was_over_url = FALSE;
+    static hotarea_t *current_url = NULL;
 
     if (!hotarea_list)
 	return FALSE;
@@ -1689,11 +1746,16 @@ check_over_url(GtkWidget *widget, GdkEvent *event, gpointer data)
 		url_cursor_over_url = gdk_cursor_new(GDK_HAND2);
 	    }
 	    if (!was_over_url) {
-		gdk_window_set_cursor(GTK_TEXT(widget)->text_area, 
+		gdk_window_set_cursor(GTK_TEXT(widget)->text_area,
 				      url_cursor_over_url);
-		balsa_gtk_html_on_url(NULL, hotarea_data->url->url);
 		was_over_url = TRUE;
 	    }
+	    if (hotarea_data != current_url) {
+		if (current_url)
+		    balsa_gtk_html_on_url(NULL, NULL);
+		balsa_gtk_html_on_url(NULL, hotarea_data->url->url);
+		current_url = hotarea_data;
+	    }
 	    return FALSE;
 	}
 	hotarea_list = g_list_next(hotarea_list);
@@ -1703,6 +1765,7 @@ check_over_url(GtkWidget *widget, GdkEvent *event, gpointer data)
 	gdk_window_set_cursor(GTK_TEXT(widget)->text_area, url_cursor_normal);
 	balsa_gtk_html_on_url(NULL, NULL);
 	was_over_url = FALSE;
+	current_url = NULL;
     }
 
     return FALSE;
@@ -1726,35 +1789,66 @@ handle_url(const message_url_t* url)
     }
 }
 
-/* if the mouse button was released over an url, try to call it */
+/* store the coordinates at which the button was pressed */
+static gint stored_x=-1, stored_y=-1;
+static GdkModifierType stored_mask=-1;
+
+static gboolean
+store_button_coords(GtkWidget *widget, GdkEventButton *event, gpointer data)
+{
+    if (event->type == GDK_BUTTON_PRESS && event->button == 1) {
+       gdk_window_get_pointer(GTK_TEXT(widget)->text_area,
+                              &stored_x, &stored_y, &stored_mask);
+       /* Take this button press out of the mask, so it won't interfere
+        * with the comparison in check_call_url()
+        * FIXME Is the mask comparison necessary?  Or should it be
+        * there, but only compare shift, ctrl, and mod1-mod5?
+        */
+       stored_mask &= ~(GDK_BUTTON_PRESS_MASK);
+    }
+    return FALSE;
+}
+
+/* if the mouse button was released over an URL, and the mouse hasn't
+ * moved since the button was pressed, try to call the URL */
 static gboolean 
 check_call_url(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
-    GList *hotarea_list = 
-	(GList *)gtk_object_get_data(GTK_OBJECT(widget), "hotarea-list");
+    GList *hotarea_list;
+    gint x, y;
+    GdkModifierType mask;
+
+    if ( !(event->type == GDK_BUTTON_RELEASE && event->button == 1) ) 
+        return FALSE;
+
+    hotarea_list = gtk_object_get_data(GTK_OBJECT(widget), "hotarea-list");
     
     if (!hotarea_list)
 	return FALSE;
 
-    if (event->type == GDK_BUTTON_RELEASE && event->button == 1) {
-	gint x;
-	gint y;
-	GdkModifierType mask;
-	
-	gdk_window_get_pointer(GTK_TEXT(widget)->text_area, &x, &y, &mask);
-	while (hotarea_list) {
-	    hotarea_t *hotarea_data = (hotarea_t *)hotarea_list->data;
+    gdk_window_get_pointer(GTK_TEXT(widget)->text_area, &x, &y, &mask);
+    /* FIXME Allow some slop here?
+     * Eg, if cursor has moved less than 2 pixels, follow the link:
+     *   if ((x >= stored_x-2) && (x <= stored_x+2) &&
+     *       (y >= stored_y-2) && (y <= stored_y+2) &&
+     *       (mask == stored_mask))
+     *
+     * Is there some gnome setting that can be used where the
+     * +/- 2 (or whatever) appears?
+     */
+    if (x == stored_x && y == stored_y && mask == stored_mask) {
+        while (hotarea_list) {
+            hotarea_t *hotarea_data = (hotarea_t *)hotarea_list->data;
 
-	    if (hotarea_data->xul <= x && x <= hotarea_data->xlr &&
-		hotarea_data->yul <= y && y <= hotarea_data->ylr) {
+            if (hotarea_data->xul <= x && x <= hotarea_data->xlr &&
+                hotarea_data->yul <= y && y <= hotarea_data->ylr) {
 
-		handle_url(hotarea_data->url);
-		break;
-	    }
-	    hotarea_list = g_list_next(hotarea_list);
-	}
+                handle_url(hotarea_data->url);
+                break;
+            }
+            hotarea_list = g_list_next(hotarea_list);
+        }
     }
-
     return FALSE;
 }
 
@@ -1783,7 +1877,7 @@ part_info_init_mimetext(BalsaMessage * bm, BalsaPartInfo * info)
     /* proper code */
     if (!libbalsa_message_body_save_temporary(info->body, NULL)) {
 	balsa_information
-	    (LIBBALSA_INFORMATION_ERROR,
+	    (LIBBALSA_INFORMATION_ERROR, NULL,
 	     _("Error writing to temporary file %s.\n"
 	       "Check the directory permissions."),
 	     info->body->temp_filename);
@@ -1791,7 +1885,7 @@ part_info_init_mimetext(BalsaMessage * bm, BalsaPartInfo * info)
     }
 
     if ((fp = fopen(info->body->temp_filename, "r")) == NULL) {
-	balsa_information(LIBBALSA_INFORMATION_ERROR,
+	balsa_information(LIBBALSA_INFORMATION_ERROR, NULL,
 			  _("Cannot open temporary file %s."),
 			  info->body->temp_filename);
 	return;
@@ -1897,7 +1991,10 @@ part_info_init_mimetext(BalsaMessage * bm, BalsaPartInfo * info)
 	    gtk_object_set_data(GTK_OBJECT(item), "url-list", 
 				(gpointer) url_list);
 	    gtk_object_set_data(GTK_OBJECT(item), "hotarea-list", 
-				(gpointer) NULL);	    
+				(gpointer) NULL);	
+            gtk_signal_connect_after(GTK_OBJECT(item), "button_press_event",
+                                     (GtkSignalFunc)store_button_coords, 
+                                     NULL);
 	    gtk_signal_connect_after(GTK_OBJECT(item), "button_release_event",
 				     (GtkSignalFunc)check_call_url, NULL);
 	    gtk_signal_connect(GTK_OBJECT(item), "motion-notify-event",
@@ -2047,7 +2144,7 @@ display_part(BalsaMessage * bm, LibBalsaMessageBody * body)
     else
 	icon_title = g_strdup_printf("(%s)", content_type);
 
-    pix = libbalsa_icon_finder(content_type, body->filename);
+    pix = libbalsa_icon_finder(content_type, body->filename, NULL);
 
     part_create_menu (info);
     pos = gnome_icon_list_append(GNOME_ICON_LIST(bm->part_list),
@@ -2078,25 +2175,96 @@ display_content(BalsaMessage * bm)
 	display_part(bm, body);
 }
 
+#ifdef HAVE_GNOME_VFS
+static void add_vfs_menu_item(BalsaPartInfo *info, 
+			      const GnomeVFSMimeApplication *app)
+{
+    gchar *menu_label = g_strdup_printf(_("Open with %s"), app->name);
+    GtkWidget *menu_item = gtk_menu_item_new_with_label (menu_label);
+    
+    gtk_object_set_data (GTK_OBJECT (menu_item), "mime_action", 
+			 g_strdup(app->id));
+    gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
+			GTK_SIGNAL_FUNC (part_context_menu_vfs_cb),
+			(gpointer) info);
+    gtk_menu_append (GTK_MENU (info->popup_menu), menu_item);
+    g_free (menu_label);
+}
+
+static gboolean in_gnome_vfs(const GnomeVFSMimeApplication *default_app, 
+			     const GList *short_list, const gchar *cmd) 
+{
+    gchar *cmd_base=g_strdup(cmd), *arg=strchr(cmd_base, '%');
+    
+    /* Note: Tries to remove the entrire argument containing %f etc., so that
+             we e.g. get rid of the whole "file:%f", not just "%f" */
+    if(arg) {
+	while(arg!=cmd && *arg!=' ')
+	    arg--;
+	
+	*arg='\0';
+    }
+    g_strstrip(cmd_base);
+    
+    if(default_app && default_app->command && strcmp(default_app->command, cmd_base)==0) {
+	g_free(cmd_base);
+	return TRUE;
+    } else {
+	const GList *item;
+
+	for(item=short_list; item; item=g_list_next(item)) {
+	    GnomeVFSMimeApplication *app=item->data;
+	    
+	    if(app->command && strcmp(app->command, cmd_base)==0) {
+		g_free(cmd_base);
+		return TRUE;
+	    }
+	}
+    }
+    g_free(cmd_base);
+    
+    return FALSE;
+}
+#endif
+
 
 static void
 part_create_menu (BalsaPartInfo* info) 
+/* Remarks: Will add items in the following order:
+            1) Default application according to GnomeVFS.
+	    2) GNOME MIME/GnomeVFS key values that don't match default
+	       application or anything on the shortlist.
+	    3) GnomeVFS shortlist applications, with the default one (sometimes
+	       included on shortlist, sometimes not) excluded. */ 
 {
     GtkWidget* menu_item;
     GList* list;
-    GList* key_list;
+    GList* key_list, *app_list;
     gchar* content_type;
     gchar* key;
     const gchar* cmd;
     gchar* menu_label;
     gchar** split_key;
     gint i;
+#ifdef HAVE_GNOME_VFS
+    GnomeVFSMimeApplication *def_app, *app;
+#endif
     
     info->popup_menu = gtk_menu_new ();
     
     content_type = libbalsa_message_body_get_content_type (info->body);
     key_list = list = gnome_mime_get_keys (content_type);
+#ifdef HAVE_GNOME_VFS
+    gdk_threads_leave(); /*FIXME: this hangs balsa */
+    app_list = gnome_vfs_mime_get_short_list_applications(content_type);
+    gdk_threads_enter(); /* FIXME: this hangs balsa */
+
+    if((def_app=gnome_vfs_mime_get_default_application(content_type))) {
+	add_vfs_menu_item(info, def_app);
+    }
+#endif
     
+
     while (list) {
         key = list->data;
 
@@ -2106,14 +2274,17 @@ part_create_menu (BalsaPartInfo* info)
 	    && (!strstr(key, "_") || strstr(key, "."))
 	    && g_strncasecmp(key, "description", 11)) {
 	    
-            if ((cmd = gnome_mime_get_value (content_type, key)) != NULL) {
+            if ((cmd = gnome_mime_get_value (content_type, key)) != NULL
+#ifdef HAVE_GNOME_VFS
+                && !in_gnome_vfs(def_app, app_list, cmd)
+#endif
+               ) {
                 if (g_strcasecmp (key, "open") == 0 || 
                     g_strcasecmp (key, "view") == 0 || 
                     g_strcasecmp (key, "edit") == 0 ||
                     g_strcasecmp (key, "ascii-view") == 0) {
-                    
                     /* uppercase first letter, make label */
-                    menu_label = g_strdup (key);
+		    menu_label = g_strdup_printf ("%s (\"%s\")", key, cmd);
                     *menu_label = toupper (*menu_label);
                 } else {
                     split_key = g_strsplit (key, ".", -1);
@@ -2126,7 +2297,6 @@ part_create_menu (BalsaPartInfo* info)
                     menu_label = g_strdup (menu_label);
                     g_strfreev (split_key);
                 }
-                
                 menu_item = gtk_menu_item_new_with_label (menu_label);
                 gtk_object_set_data (GTK_OBJECT (menu_item), "mime_action", 
                                      key);
@@ -2140,6 +2310,22 @@ part_create_menu (BalsaPartInfo* info)
         list = g_list_next (list);
     }
 
+#ifdef HAVE_GNOME_VFS
+    list=app_list;
+
+    while (list) {
+	app=list->data;
+
+	if(app && (!def_app || strcmp(app->name, def_app->name)!=0)) {
+	    add_vfs_menu_item(info, app);
+	}
+
+        list = g_list_next (list);
+    }
+    gnome_vfs_mime_application_free(def_app);
+#endif
+    
+
     menu_item = gtk_menu_item_new_with_label (_("Save..."));
     gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
                         GTK_SIGNAL_FUNC (part_context_menu_save),
@@ -2149,6 +2335,9 @@ part_create_menu (BalsaPartInfo* info)
     gtk_widget_show_all (info->popup_menu);
 
     g_list_free (key_list);
+#ifdef HAVE_GNOME_VFS
+    gnome_vfs_mime_application_list_free (app_list);
+#endif
     g_free (content_type);
 }
 
@@ -2275,7 +2464,7 @@ part_context_menu_cb(GtkWidget * menu_item, BalsaPartInfo * info)
 
     if (key != NULL && (cmd = gnome_mime_get_value(content_type, key)) != NULL) {
 	if (!libbalsa_message_body_save_temporary(info->body, NULL)) {
-	    balsa_information(LIBBALSA_INFORMATION_WARNING,
+	    balsa_information(LIBBALSA_INFORMATION_WARNING, NULL,
 			      _("could not create temporary file %s"),
 			      info->body->temp_filename);
 	    g_free(content_type);
@@ -2298,6 +2487,40 @@ part_context_menu_cb(GtkWidget * menu_item, BalsaPartInfo * info)
 }
 
 
+#ifdef HAVE_GNOME_VFS
+static void
+part_context_menu_vfs_cb(GtkWidget * menu_item, BalsaPartInfo * info)
+{
+    gchar *id;
+    
+    if(id = gtk_object_get_data (GTK_OBJECT (menu_item), "mime_action")) {
+	GnomeVFSMimeApplication *app=
+	    gnome_vfs_mime_application_new_from_id(id);
+	if(app) {
+	    if (libbalsa_message_body_save_temporary(info->body, NULL)) {
+		gchar *exe_str=
+		    g_strdup_printf("%s %s%s", app->command,
+				    (app->expects_uris==GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS?"file:":""),
+				    info->body->temp_filename);
+		
+		gnome_execute_shell(NULL, exe_str);
+		fprintf(stderr, "Executed: %s\n", exe_str);
+		g_free (exe_str);
+	    } else {
+		balsa_information(LIBBALSA_INFORMATION_WARNING, NULL,
+				  _("could not create temporary file %s"),
+				  info->body->temp_filename);
+	    }
+	    gnome_vfs_mime_application_free(app);    
+	} else {
+	    fprintf(stderr, "lookup for application %s returned NULL\n", id);
+	}
+    }
+}
+#endif
+
+
+
 void
 balsa_message_next_part(BalsaMessage * bmessage)
 {
@@ -2430,8 +2653,7 @@ static void add_multipart(BalsaMessage *bm, LibBalsaMessageBody *parent)
 	    if(body) {
 		add_body(bm, body);
 		for(body=body->next; body; body=body->next) {
-		    if(body->mutt_body && 
-		       body->mutt_body->disposition==DISPINLINE)
+                    if(libbalsa_message_body_is_inline(body))
 			add_body(bm, body);
 		}
 	    }
@@ -2656,7 +2878,7 @@ balsa_gtk_html_link_clicked(GtkWidget *html, const gchar *url)
 static void
 balsa_gtk_html_on_url(GtkWidget *html, const gchar *url)
 {
-    static url_pushed = FALSE;
+    static gboolean url_pushed = FALSE;
 
     if( url ) {
 	if (url_pushed) {
@@ -2879,7 +3101,6 @@ static LibBalsaMessage *create_mdn_reply (LibBalsaMessage *for_msg,
     gchar *date, *dummy;
 
     /* create a message with the header set from the incoming message */
-    libbalsa_set_charset("ISO-8859-1");  /* how do I detect the *standard* setting? */
     message = libbalsa_message_new();
     dummy = libbalsa_address_to_gchar(balsa_app.current_ident->address, 0);
     message->from = libbalsa_address_new_from_string(dummy);
