@@ -30,8 +30,8 @@
 
 typedef struct {
     GnomeDialog *dialog;
-    GtkWidget * folder_name, *server, *port, *username, *password, 
-	*subscribed, *list_inbox, *prefix;
+    GtkWidget * folder_name, *server, *port, *username, *remember,
+        *password, *subscribed, *list_inbox, *prefix;
 #ifdef USE_SSL
     GtkWidget *use_ssl;
 #endif
@@ -67,26 +67,43 @@ static void imap_use_ssl_cb(GtkToggleButton * button,
 static void
 imap_use_ssl_cb(GtkToggleButton * button, FolderDialogData * fcw)
 {
-    gint zero = 0;
-    GtkEditable *port = GTK_EDITABLE(fcw->port);
-    gboolean use_ssl = gtk_toggle_button_get_active(button);
-    gtk_editable_delete_text(port, 0, -1);
-    gtk_editable_insert_text(port, use_ssl ? "993" : "143", 3, &zero);
+    char *colon, *newhost;
+    const gchar* host = gtk_entry_get_text(GTK_ENTRY(fcw->server));
+    gchar* port = gtk_toggle_button_get_active(button) ? "993" : "143";
+
+    if( (colon=strchr(host,':')) != NULL) 
+        *colon = '\0';
+    newhost = g_strconcat(host, ":", port, NULL);
+    gtk_entry_set_text(GTK_ENTRY(fcw->server), newhost);
+    g_free(newhost);
 }
 #endif
 
+static void
+remember_cb(GtkToggleButton * button, FolderDialogData * fcw)
+{
+    gtk_widget_set_sensitive(fcw->password,
+                             gtk_toggle_button_get_active(button));
+}
+
 static void
 folder_conf_clicked_cb(GtkObject* dialog, int buttonno, gpointer data)
 {
     FolderDialogData *fcw = (FolderDialogData*)data;
     LibBalsaServer * s = fcw->mn ? fcw->mn->server : NULL;
+#if BALSA_MAJOR < 2
     static GnomeHelpMenuEntry help_entry = { NULL, "folder-config.html" };
+#else
+    static const gchar help_path[] = "folder-config.html";
+    GError *err = NULL;
+#endif                          /* BALSA_MAJOR < 2 */
     gboolean insert;
-    gint port_no;
     GNode *gnode;
 
+#if BALSA_MAJOR < 2
     help_entry.name = gnome_app_id;
 
+#endif                          /* BALSA_MAJOR < 2 */
     switch(buttonno) {
     case 0: /* OK */
 	if(!fcw->mn) { 
@@ -95,16 +112,16 @@ folder_conf_clicked_cb(GtkObject* dialog, int buttonno, gpointer data)
 	}
 	else insert = FALSE;
 	
-	port_no = atoi(gtk_entry_get_text(GTK_ENTRY(fcw->port)));
 	libbalsa_server_set_host(s, 
-                                 gtk_entry_get_text(GTK_ENTRY(fcw->server)), 
-				 port_no
+                                 gtk_entry_get_text(GTK_ENTRY(fcw->server))
 #ifdef USE_SSL
 				 , gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->use_ssl))
 #endif
                                  );
 	libbalsa_server_set_username
 	    (s, gtk_entry_get_text(GTK_ENTRY(fcw->username)));
+        s->remember_passwd = 
+            gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fcw->remember));
 	libbalsa_server_set_password
 	    (s, gtk_entry_get_text(GTK_ENTRY(fcw->password)));
 	
@@ -137,7 +154,16 @@ folder_conf_clicked_cb(GtkObject* dialog, int buttonno, gpointer data)
         gnome_dialog_close(GNOME_DIALOG(dialog));
         break;
     case 2:
+#if BALSA_MAJOR < 2
         gnome_help_display(NULL, &help_entry);
+#else
+        gnome_help_display_uri(help_path, &err);
+        if (err) {
+            g_print(_("Error displaying %s: %s\n"), help_path,
+                    err->message);
+            g_error_free(err);
+        }
+#endif                          /* BALSA_MAJOR < 2 */
         break;
     }
 }
@@ -153,6 +179,7 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
     FolderDialogData fcw;
     guint keyval;
     LibBalsaServer * s = mn ? mn->server : NULL;
+    gchar *default_server = libbalsa_guess_imap_server();
 
     fcw.mn = mn;
     fcw.dialog = GNOME_DIALOG(gnome_dialog_new(_("Remote IMAP folder"), 
@@ -168,55 +195,54 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
     frame = gtk_frame_new(_("Remote IMAP folder set"));
     gtk_box_pack_start(GTK_BOX(fcw.dialog->vbox),
                        frame, TRUE, TRUE, 0);
-    table = gtk_table_new(8, 2, FALSE);
+    table = gtk_table_new(9, 2, FALSE);
     gtk_container_add(GTK_CONTAINER(frame), table);
  
     /* INPUT FIELD CREATION */
     create_label(_("Descriptive _Name:"), table, 0, &keyval);
-    fcw.folder_name = create_entry(fcw.dialog, table, validate_folder, &fcw, 
-				   0, mn ? mn->name : NULL, 
+    fcw.folder_name = create_entry(fcw.dialog, table,
+                                   GTK_SIGNAL_FUNC(validate_folder),
+                                   &fcw, 0, mn ? mn->name : NULL, 
 				   keyval);
 
     create_label(_("_Server:"), table, 1, &keyval);
-    fcw.server = create_entry(fcw.dialog, table, validate_folder, &fcw, 1, 
-			  s ? s->host : "localhost",
-			  keyval);
-
-    create_label(_("_Port:"), table, 2, &keyval);
-    if(s) {
-	gchar* tmp = g_strdup_printf("%d", s->port);
-	fcw.port = create_entry(fcw.dialog, table, NULL, NULL, 2, tmp, keyval);
-	g_free(tmp);
-    } else fcw.port = 
-	       create_entry(fcw.dialog, table, NULL, NULL, 2, "143", keyval);
-
+    fcw.server = create_entry(fcw.dialog, table,
+                              GTK_SIGNAL_FUNC(validate_folder),
+                              &fcw, 1, s ? s->host : default_server,
+			      keyval);
 
     create_label(_("_User name:"), table, 3, &keyval);
-    fcw.username = create_entry(fcw.dialog, table, validate_folder, &fcw, 3, 
-			    s ? s->user : g_get_user_name(), 
-			    keyval);
+    fcw.username = create_entry(fcw.dialog, table,
+                                GTK_SIGNAL_FUNC(validate_folder),
+                                &fcw, 3, s ? s->user : g_get_user_name(), 
+			        keyval);
 
-    create_label(_("_Password:"), table, 4, &keyval);
-    fcw.password = create_entry(fcw.dialog, table, NULL, NULL, 4,
-				s ? s->passwd : NULL, 
-				keyval);
+    fcw.remember = create_check(fcw.dialog, _("_Remember password"), 
+                                table, 4, s ? s->remember_passwd : TRUE);
+    gtk_signal_connect(GTK_OBJECT(fcw.remember), "toggled",
+                       GTK_SIGNAL_FUNC(remember_cb), 
+                       &fcw);
+
+    create_label(_("_Password:"), table, 5, &keyval);
+    fcw.password = create_entry(fcw.dialog, table, NULL, NULL, 5,
+				s ? s->passwd : NULL, keyval);
     gtk_entry_set_visibility(GTK_ENTRY(fcw.password), FALSE);
 
     fcw.subscribed = create_check(fcw.dialog, _("_Subscribed folders only"), 
-                                  table, 5, mn ? mn->subscribed : FALSE);
+                                  table, 6, mn ? mn->subscribed : FALSE);
     fcw.list_inbox = create_check(fcw.dialog, _("_Always show INBOX"), 
-                                  table, 6, mn ? mn->list_inbox : TRUE); 
+                                  table, 7, mn ? mn->list_inbox : TRUE); 
 
-    create_label(_("_Prefix"), table, 7, &keyval);
-    fcw.prefix = create_entry(fcw.dialog, table, NULL, NULL, 7, 
+    create_label(_("_Prefix"), table, 8, &keyval);
+    fcw.prefix = create_entry(fcw.dialog, table, NULL, NULL, 8,
 			      mn ? mn->dir : NULL, keyval);
     
 #ifdef USE_SSL
     fcw.use_ssl = create_check(fcw.dialog,
 			       _("Use SSL (IMAPS)"),
-			       table, 8, s ? s->use_ssl : FALSE);
-    gtk_signal_connect(GTK_OBJECT(fcw.use_ssl), "toggled", imap_use_ssl_cb, 
-                       &fcw);
+			       table, 9, s ? s->use_ssl : FALSE);
+    gtk_signal_connect(GTK_OBJECT(fcw.use_ssl), "toggled",
+                       GTK_SIGNAL_FUNC(imap_use_ssl_cb), &fcw);
 #endif
 
     gtk_widget_show_all(GTK_WIDGET(fcw.dialog));
@@ -225,7 +251,7 @@ folder_conf_imap_node(BalsaMailboxNode *mn)
     gtk_widget_grab_focus(fcw.folder_name);
 
     gtk_signal_connect(GTK_OBJECT(fcw.dialog), "clicked", 
-                       folder_conf_clicked_cb, &fcw);
+                       GTK_SIGNAL_FUNC(folder_conf_clicked_cb), &fcw);
     gnome_dialog_run(fcw.dialog);
 }
 
@@ -239,13 +265,15 @@ typedef struct {
     GnomeDialog *dialog;
     GtkWidget *parent_folder, *folder_name;
     gchar *old_folder, *old_parent;
-    BalsaMailboxNode *mbnode;
+    BalsaMailboxNode *parent; /* (new) parent of the mbnode.  */
+                              /* Used for renaming and creation */
+    BalsaMailboxNode *mbnode; /* edited mbnode          */
 } SubfolderDialogData;
 
 static void
 validate_sub_folder(GtkWidget * w, SubfolderDialogData * fcw)
 {
-    BalsaMailboxNode *mn = fcw->mbnode;
+    BalsaMailboxNode *mn = fcw->parent;
     /*
      * Allow typing in the parent_folder entry box only if we already
      * have the server information in mn:
@@ -259,8 +287,8 @@ validate_sub_folder(GtkWidget * w, SubfolderDialogData * fcw)
      * will deny permission:
      */
     gnome_dialog_set_sensitive(fcw->dialog, 0, have_server &&
-			       *gtk_entry_get_text(GTK_ENTRY
-						   (fcw->folder_name)));
+                               *gtk_entry_get_text(GTK_ENTRY
+                                                   (fcw->folder_name)));
 }
 
 /* callbacks for a `Browse...' button: */
@@ -274,7 +302,7 @@ browse_button_select_row_cb(GtkCTree * ctree, GList * node, gint column,
     if (mbnode) {
 	SubfolderDialogData *fcw = (SubfolderDialogData *) data;
 	gchar *path = mbnode->dir;
-	fcw->mbnode = mbnode;
+        fcw->parent = mbnode;
 	if (path)
 	    gtk_entry_set_text(GTK_ENTRY(fcw->parent_folder), path);
 	gnome_dialog_close(GNOME_DIALOG
@@ -334,20 +362,26 @@ browse_button_cb(GtkWidget * widget, gpointer data)
 
     gtk_widget_show(ctree);
     gtk_widget_show(scroll);
-    gnome_dialog_run(GNOME_DIALOG(dialog));
+    gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
 }
 
 static void
 subfolder_conf_clicked_cb(GtkObject* dialog, int buttonno, gpointer data)
 {
+#if BALSA_MAJOR < 2
     static GnomeHelpMenuEntry help_entry =
 	{ NULL, "folder-config.html#SUBFOLDER-CONFIG" };
+#else
+    static const gchar help_path[] = "folder-config.html#SUBFOLDER-CONFIG";
+    GError *err = NULL;
+#endif                          /* BALSA_MAJOR < 2 */
     SubfolderDialogData *fcw = (SubfolderDialogData*)data;
-    gint button;
     gchar* parent, *folder;
 
+#if BALSA_MAJOR < 2
     help_entry.name = gnome_app_id;
 
+#endif                          /* BALSA_MAJOR < 2 */
     switch(buttonno) {
     case 0: /* OK */
 	parent = 
@@ -355,7 +389,7 @@ subfolder_conf_clicked_cb(GtkObject* dialog, int buttonno, gpointer data)
 	folder = 
             gtk_editable_get_chars(GTK_EDITABLE(fcw->folder_name), 0, -1);
         
-	if (fcw->old_folder) {
+	if (fcw->mbnode) {
 	    /* rename */
 	    if (strcmp(parent, fcw->old_parent) || 
                 strcmp(folder, fcw->old_folder)) {
@@ -385,24 +419,30 @@ subfolder_conf_clicked_cb(GtkObject* dialog, int buttonno, gpointer data)
 		     */
 		    balsa_window_close_mbnode(balsa_app.main_window, 
                                               fcw->mbnode);
-		    libbalsa_imap_rename_subfolder(fcw->mbnode->dir, 
-                                                   parent, folder,
-						   fcw->mbnode->subscribed,
-						   fcw->mbnode->server);
+		    libbalsa_imap_rename_subfolder
+                        (LIBBALSA_MAILBOX_IMAP(fcw->mbnode->mailbox),
+                         parent, folder, fcw->mbnode->subscribed);
+                    g_free(fcw->mbnode->dir);
+                    fcw->mbnode->dir = g_strdup(parent);
 
-		    /*	Rescan as little of the tree as possible.
-		     *	We'll assume that `parent' is consistent
-		     *	with `fcw->mbnode' (that is, the parent path
-		     *	was found by browsing, not typing in the entry.
-		     */
-		    if (!strncmp(parent, fcw->old_parent, strlen(parent)))
+		    /*	Rescan as little of the tree as possible. */
+		    if (!strncmp(parent, fcw->old_parent, strlen(parent))){
 			/* moved it up the tree */
-			balsa_mailbox_node_rescan(fcw->mbnode);
-		    else if (!strncmp(parent, fcw->old_parent, 
-                                      strlen(fcw->old_parent)))
+                        GNode* n = balsa_find_dir(balsa_app.mailbox_nodes,
+                                                  parent);
+                        if(n)
+                            balsa_mailbox_node_rescan
+                                (BALSA_MAILBOX_NODE(n->data));
+                        else printf("Parent not found!?\n");
+                    } else if (!strncmp(parent, fcw->old_parent, 
+                                        strlen(fcw->old_parent))) {
 			/* moved it down the tree */
-			balsa_mailbox_node_rescan(fcw->mbnode->parent);
-		    else {
+                        GNode* n = balsa_find_dir(balsa_app.mailbox_nodes,
+                                                  fcw->old_parent);
+                        if(n)
+                            balsa_mailbox_node_rescan
+                                (BALSA_MAILBOX_NODE(n->data));
+                    } else {
 			/* moved it sideways: a chain of folders might
 			 * go away, so we'd better rescan from higher up
 			 */
@@ -415,13 +455,13 @@ subfolder_conf_clicked_cb(GtkObject* dialog, int buttonno, gpointer data)
 		}
 	    }
 	} else {
-	    /* create */
+	    /* create and subscribe, if parent was. */
 	    libbalsa_imap_new_subfolder(parent, folder,
-					fcw->mbnode->subscribed,
-					fcw->mbnode->server);
+					fcw->parent->subscribed,
+					fcw->parent->server);
 
 	    /* see it as server sees it: */
-	    balsa_mailbox_node_rescan(fcw->mbnode);
+	    balsa_mailbox_node_rescan(fcw->parent);
 	}
 	g_free(parent);
 	g_free(folder);
@@ -430,7 +470,17 @@ subfolder_conf_clicked_cb(GtkObject* dialog, int buttonno, gpointer data)
         gnome_dialog_close(GNOME_DIALOG(dialog));
         break;
     case 2:
+#if BALSA_MAJOR < 2
         gnome_help_display(NULL, &help_entry);
+#else
+        gnome_help_display_uri(help_path, &err);
+        if (err) {
+            g_print(_("Error displaying %s: %s\n"), help_path,
+                    err->message);
+            g_error_free(err);
+        }
+#endif                          /* BALSA_MAJOR < 2 */
+        break;
     }
 }
 
@@ -445,7 +495,6 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
     GtkWidget *frame, *table, *subtable, *browse_button;
     SubfolderDialogData fcw;
     guint keyval;
-    gint button;
 
     if (mn) {
 	/* update */
@@ -455,7 +504,8 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
 		"has no properties that can be changed."))));
 	    return;
 	}
-	fcw.mbnode = mn->parent;
+	fcw.mbnode = mn;
+	fcw.parent = mn->parent;
 	fcw.old_folder = mn->mailbox->name;
     } else {
 	/* create */
@@ -465,9 +515,10 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
 	    fcw.mbnode = mbnode;
 	else
 	    fcw.mbnode = NULL;
-	fcw.old_folder = NULL;
+        fcw.old_folder = NULL;
+        fcw.parent = NULL;
     }
-    fcw.old_parent = fcw.mbnode ? fcw.mbnode->dir : NULL;
+    fcw.old_parent = fcw.mbnode ? fcw.mbnode->parent->dir : NULL;
 
     fcw.dialog = GNOME_DIALOG(gnome_dialog_new(_("Remote IMAP subfolder"), 
 					       mn ? _("Update") : _("Create"), 
@@ -490,12 +541,14 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
  
     /* INPUT FIELD CREATION */
     create_label(_("_Folder name:"), table, 0, &keyval);
-    fcw.folder_name = create_entry(fcw.dialog, table, validate_sub_folder,
+    fcw.folder_name = create_entry(fcw.dialog, table,
+                                   GTK_SIGNAL_FUNC(validate_sub_folder),
 				   &fcw, 0, fcw.old_folder, keyval);
 
     subtable = gtk_table_new(1, 3, FALSE);
     create_label(_("_Subfolder of:"), table, 1, &keyval);
-    fcw.parent_folder = create_entry(fcw.dialog, subtable, validate_sub_folder,
+    fcw.parent_folder = create_entry(fcw.dialog, subtable,
+                                     GTK_SIGNAL_FUNC(validate_sub_folder),
 				     &fcw, 0, fcw.old_parent, keyval);
 
     browse_button = gtk_button_new_with_label(_("Browse..."));
@@ -513,7 +566,7 @@ folder_conf_imap_sub_node(BalsaMailboxNode * mn)
     gtk_widget_grab_focus(fcw.folder_name);
 
     gtk_signal_connect(GTK_OBJECT(fcw.dialog), "clicked", 
-                       subfolder_conf_clicked_cb, &fcw);
+                       GTK_SIGNAL_FUNC(subfolder_conf_clicked_cb), &fcw);
     gnome_dialog_run(fcw.dialog);
 }
 
