@@ -30,6 +30,7 @@ enum _BalsaInformationShow {
     BALSA_INFORMATION_SHOW_STDERR,
 };
 
-void balsa_information(LibBalsaInformationType type, const char *fmt, ...);
+void balsa_information(LibBalsaInformationType type, GtkWindow* parent,
+                       const char *fmt, ...);
 
 #endif
