@@ -29,6 +29,7 @@
 #include "mailbox-node.h"
 #include "save-restore.h"
 #include "notify.h"
+#include "filter.h"
 
 /* MailboxNode object is a GUI representation of a mailbox, or entire 
    set of them. It can read itself from the configuration, save its data,
@@ -51,8 +52,10 @@ static void balsa_mailbox_node_real_load_config(BalsaMailboxNode* mn,
 static GNode* add_local_mailbox(GNode*root, const char*d_name, const char* fn);
 static GNode* add_local_folder(GNode*root, const char*d_name, const char* fn);
 
-static GNode* add_imap_mailbox(GNode*root, const char* fn, char delim);
-static GNode* add_imap_folder(GNode*root, const char* fn, char delim);
+static GNode *add_imap_mailbox(GNode * root, const char *fn, char delim,
+			       gboolean scanned);
+static GNode *add_imap_folder(GNode * root, const char *fn, char delim,
+			      gboolean scanned);
 
 enum {
     SAVE_CONFIG,
@@ -142,6 +145,7 @@ balsa_mailbox_node_init(BalsaMailboxNode * mn)
     mn->sort_type = GTK_SORT_DESCENDING;
     mn->sort_field = BALSA_SORT_NO;
     mn->subscribed = FALSE;
+    mn->scanned = FALSE;
 }
 
 static void
@@ -229,13 +233,17 @@ static void
 imap_dir_cb(BalsaMailboxNode* mb, GNode* r)
 {
     g_return_if_fail(mb->server);
+    gdk_threads_leave();
     libbalsa_scanner_imap_dir(r, mb->server, mb->dir, mb->subscribed,
-                              mb->list_inbox, 7,
+                              mb->list_inbox, 
+                              balsa_app.imap_scan_depth,
 			      add_imap_folder, add_imap_mailbox);
+    gdk_threads_enter();
     /* register whole tree */
-    printf("imap_dir_cb:  main mailbox node %s mailbox is %p\n", 
-	   BALSA_MAILBOX_NODE(r->data)->name, 
-	   BALSA_MAILBOX_NODE(r->data)->mailbox);
+    if(BALSA_MAILBOX_NODE(r->data)->name)
+        printf("imap_dir_cb:  main mailbox node %s mailbox is %p\n", 
+               BALSA_MAILBOX_NODE(r->data)->name, 
+               BALSA_MAILBOX_NODE(r->data)->mailbox);
     g_node_traverse(r, G_IN_ORDER, G_TRAVERSE_ALL, -1,
 		    (GNodeTraverseFunc) register_mailbox, NULL);
 }
@@ -290,24 +298,21 @@ balsa_mailbox_node_new_from_config(const gchar* prefix)
     /* take over the ownership */
     gtk_object_ref(GTK_OBJECT(folder->server)); 
     gtk_object_sink(GTK_OBJECT(folder->server)); 
-    printf("Loading its configuration...\n");
-    libbalsa_server_load_config(folder->server, 143);
+    libbalsa_server_load_config(folder->server);
 
 #ifdef USE_SSL  
-    printf("Server loaded, host: %s, port %d %s\n", folder->server->host,
-	   folder->server->port,
+    printf("Server loaded, host: %s, %s.\n", folder->server->host,
 	   folder->server->use_ssl ? "SSL" : "no SSL");
 #else
-    printf("Server loaded, host: %s, port %d\n", folder->server->host,
-	   folder->server->port);
+    printf("Server loaded, host: %s\n", folder->server->host);
 #endif
-    gtk_signal_connect(GTK_OBJECT(folder->server),
-		       "get-password", GTK_SIGNAL_FUNC(ask_password),
-		       NULL);
     gtk_signal_connect(GTK_OBJECT(folder), "show-prop-dialog", 
 		       folder_conf_imap_node, NULL);
     gtk_signal_connect(GTK_OBJECT(folder), "append-subtree", 
 		       imap_dir_cb, NULL);
+    gtk_signal_connect(GTK_OBJECT(folder->server),
+                       "get-password", GTK_SIGNAL_FUNC(ask_password),
+                       NULL);
     balsa_mailbox_node_load_config(folder, prefix);
 
     folder->dir = gnome_config_get_string("Directory");
@@ -406,6 +411,27 @@ balsa_mailbox_node_save_config(BalsaMailboxNode* mn, const gchar* prefix)
 		    balsa_mailbox_node_signals[SAVE_CONFIG], prefix);
 }
 
+void
+balsa_mailbox_local_rescan_parent(LibBalsaMailbox* mbx)
+{
+    gchar *dir; 
+    GNode* parent = NULL;
+
+    g_return_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(mbx));
+
+    for(dir = g_strdup(libbalsa_mailbox_local_get_path(mbx));
+        strlen(dir)>1 /* i.e dir != "/" */ &&
+            !(parent = balsa_find_dir(balsa_app.mailbox_nodes,dir));
+        ) {
+        gchar* tmp =  g_dirname(dir); g_free(dir);
+        dir = tmp;
+    }
+    if(parent)
+        balsa_mailbox_node_rescan(BALSA_MAILBOX_NODE(parent->data)); 
+    else g_warning("parent for %s not found.\n", mbx->name);
+    g_free(dir);
+}
+
 /* balsa_mailbox_node_rescan:
    rescans given folders. It can be called on configuration update or just
    to discover mailboxes that did not exist when balsa was initially 
@@ -418,8 +444,6 @@ void balsa_mailbox_node_rescan(BalsaMailboxNode* mn)
 {
     GNode *gnode;
 
-    g_return_if_fail(mn->mailbox == NULL);
-
     gnode = balsa_find_mbnode(balsa_app.mailbox_nodes, mn);
 
     if(gnode) {
@@ -430,6 +454,9 @@ void balsa_mailbox_node_rescan(BalsaMailboxNode* mn)
 	balsa_mailbox_node_append_subtree(mn, gnode);
 	mn->expanded = expanded;
 	balsa_mblist_repopulate(balsa_app.mblist);
+        if (expanded)
+            /* if this is an IMAP node, we must scan the children */
+	    mblist_scan_mailbox_node(balsa_app.mblist, mn);
     } else g_warning("folder node %s (%p) not found in hierarchy.\n",
 		     mn->name, mn);
 }
@@ -541,6 +568,17 @@ mb_rescan_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
     balsa_mailbox_node_rescan(mbnode);
 }
 
+static void
+mb_filter_cb(GtkWidget * widget, BalsaMailboxNode * mbnode)
+{
+    if (mbnode->mailbox) filters_run_dialog(mbnode->mailbox);
+    else
+	/* FIXME : Perhaps should we be able to apply filters on folders (ie recurse on all mailboxes in it),
+	   but there are problems of infinite recursion (when one mailbox being filtered is also the destination
+	   of the filter action (eg a copy)). So let's see that later :) */
+	g_print("You can apply filters only on mailbox\n");
+}
+
 GtkWidget *
 balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
 {
@@ -551,6 +589,9 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
     /*  g_return_val_if_fail(mailbox != NULL, NULL); */
 
     menu = gtk_menu_new();
+    /* it's a single-use menu, so we must destroy it when we're done */
+    gtk_signal_connect(GTK_OBJECT(menu), "selection-done",
+                       gtk_object_destroy, NULL);
 
     submenu = gtk_menu_new();
     add_menu_entry(submenu, _("Local mbox mailbox..."),  
@@ -574,9 +615,11 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
     
     gtk_menu_append(GTK_MENU(menu), menuitem);
     
-    if(mbnode == NULL) /* clicked on the empty space */
+    if(mbnode == NULL) {/* clicked on the empty space */
+        add_menu_entry(menu, _("Rescan"), mb_rescan_cb, 
+                       BALSA_MAILBOX_NODE(balsa_app.mailbox_nodes->data));
 	return menu;
-
+    }
     /* If we didn't click on a mailbox node then there is only one option. */
     add_menu_entry(menu, NULL, NULL, mbnode);
 
@@ -603,14 +646,14 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
 	    add_menu_entry(menu, NULL, NULL, mbnode);
 	}
 	
-	if(mbnode->mailbox != balsa_app.inbox)
-	    add_menu_entry(menu, _("Mark as Inbox"),    mb_inbox_cb,    mbnode);
-	if(mbnode->mailbox != balsa_app.sentbox)
-	    add_menu_entry(menu, _("Mark as Sentbox"),  mb_sentbox_cb,  mbnode);
-	if(mbnode->mailbox != balsa_app.trash)
-	    add_menu_entry(menu, _("Mark as Trash"),    mb_trash_cb,    mbnode);
-	if(mbnode->mailbox != balsa_app.draftbox)
-	    add_menu_entry(menu, _("Mark as Draftbox"), mb_draftbox_cb, mbnode);
+	if(!BALSA_IS_MAILBOX_SPECIAL(mbnode->mailbox)) {
+	    add_menu_entry(menu, _("Mark as Inbox"),    mb_inbox_cb,   mbnode);
+	    add_menu_entry(menu, _("Mark as Sentbox"),  mb_sentbox_cb, mbnode);
+	    add_menu_entry(menu, _("Mark as Trash"),    mb_trash_cb,   mbnode);
+	    add_menu_entry(menu, _("Mark as Draftbox"), mb_draftbox_cb,mbnode);
+        }
+	/* FIXME : No test on mailbox type is made yet, should we ? */
+	add_menu_entry(menu, _("Edit/Apply filters"), mb_filter_cb, mbnode);
     } else {
 	add_menu_entry(menu, _("Rescan"),   mb_rescan_cb,   mbnode);
     }
@@ -630,79 +673,10 @@ balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode)
    to the configuration).
 */
 
-static gboolean
-traverse_find_path(GNode * node, gpointer * d)
-{
-    const gchar *path;
-    LibBalsaMailbox * mailbox;
-    if(node->data == NULL) /* true for root node only */
-	return FALSE;
-    
-    mailbox = ((BalsaMailboxNode *) node->data)->mailbox;
-
-    if(LIBBALSA_IS_MAILBOX_LOCAL(mailbox)) 
-	path = libbalsa_mailbox_local_get_path(mailbox);
-    else if(!mailbox)
-	path = ((BalsaMailboxNode *) node->data)->name;
-    else 
-	return FALSE;
-
-    if (strcmp(path, (gchar *) d[0]))
-	return FALSE;
-
-    d[1] = node;
-    return TRUE;
-}
-
-static GNode *
-find_by_path(GNode * root, GTraverseType order, GTraverseFlags flags,
-	     const gchar * path)
-{
-    gpointer d[2];
-
-    d[0] = (gchar *) path;
-    d[1] = NULL;
-    g_node_traverse(root, order, flags, -1,
-		    (GNodeTraverseFunc) traverse_find_path, d);
-
-    return d[1];
-}
-
-static gboolean
-traverse_find_url(GNode * node, gpointer * d)
-{
-    LibBalsaMailbox * mailbox;
-    if(node->data == NULL) /* true for root node only */
-	return FALSE;
-    
-    mailbox = ((BalsaMailboxNode *) node->data)->mailbox;
-    if (mailbox == NULL || strcmp(mailbox->url, (gchar *) d[0]))
-	return FALSE;
-
-    d[1] = node;
-    return TRUE;
-}
-
-static GNode *
-find_by_url(GNode * root, GTraverseType order, GTraverseFlags flags,
-	     const gchar * path)
-{
-    gpointer d[2];
-
-    d[0] = (gchar *) path;
-    d[1] = NULL;
-    g_node_traverse(root, order, flags, -1,
-		    (GNodeTraverseFunc) traverse_find_url, d);
-
-    return d[1];
-}
-
-
 static GNode*
 remove_mailbox_from_nodes(LibBalsaMailbox* mailbox)
 {
-    GNode* gnode = find_by_url(balsa_app.mailbox_nodes, G_LEVEL_ORDER, 
-			       G_TRAVERSE_ALL, mailbox->url);
+    GNode* gnode = balsa_find_url(balsa_app.mailbox_nodes, mailbox->url);
     if (gnode)
 	g_node_unlink(gnode);
     else
@@ -780,8 +754,7 @@ GNode* add_local_folder(GNode*root, const char*d_name, const char* path)
 	return NULL;
 
     /* don't add if the folder is already in the configuration */
-    if (find_by_path(balsa_app.mailbox_nodes, G_LEVEL_ORDER, 
-		     G_TRAVERSE_ALL, path))
+    if (balsa_find_path(balsa_app.mailbox_nodes, path))
 	return NULL;
 
     BALSA_MAILBOX_NODE(node->data)->parent = (BalsaMailboxNode*)root->data;
@@ -809,7 +782,7 @@ get_parent_folder_name(const gchar* path, char delim)
  * Finds the node if it exists, and creates one if it doesn't.
  */
 static GNode* 
-add_imap_entry(GNode*root, const char* fn, char delim)
+add_imap_entry(GNode*root, const char* fn, char delim, gboolean scanned)
 { 
     GNode* node;
     gchar * parent_name;
@@ -818,26 +791,26 @@ add_imap_entry(GNode*root, const char* fn, char delim)
     const gchar *basename;
     gchar* url;
 
-    node = balsa_app_find_by_dir(root, fn);
+    node = balsa_find_dir(root, fn);
     if (node && node != root)
 	return node;
 
     parent_name = get_parent_folder_name(fn, delim);
-    parent = balsa_app_find_by_dir(root, parent_name);
+    parent = balsa_find_dir(root, parent_name);
     if (parent == NULL)
         parent = root;
     g_free(parent_name);
 
     g_return_val_if_fail(parent, NULL);
 
-    url = g_strdup_printf("imap%s://%s:%i/%s",
+    url = g_strdup_printf("imap%s://%s/%s",
 #ifdef USE_SSL
-			  BALSA_MAILBOX_NODE(root->data)->server->use_ssl ? "s" : "",
+			  BALSA_MAILBOX_NODE(root->data)->server->use_ssl 
+                          ? "s" : "",
 #else
 			  "",
 #endif
 			  BALSA_MAILBOX_NODE(root->data)->server->host,
-			  BALSA_MAILBOX_NODE(root->data)->server->port,
 			  fn);
     node = remove_special_mailbox_by_url(url);
     g_free(url);
@@ -853,6 +826,7 @@ add_imap_entry(GNode*root, const char* fn, char delim)
 	mbnode->name = g_strdup(basename);
 	mbnode->parent = BALSA_MAILBOX_NODE(parent->data);
 	mbnode->subscribed = mbnode->parent->subscribed;
+	mbnode->scanned = scanned;
 	node = g_node_new(mbnode);
     }
 
@@ -873,7 +847,7 @@ add_imap_entry(GNode*root, const char* fn, char delim)
    
 */
 static GNode*
-add_imap_mailbox(GNode*root, const char* fn, char delim)
+add_imap_mailbox(GNode*root, const char* fn, char delim, gboolean scanned)
 { 
     LibBalsaMailboxImap* m;
     const gchar *basename;
@@ -888,7 +862,7 @@ add_imap_mailbox(GNode*root, const char* fn, char delim)
 	    return NULL; 
     }
 
-    node = add_imap_entry(root, fn, delim);
+    node = add_imap_entry(root, fn, delim, scanned);
     mbnode = BALSA_MAILBOX_NODE(node->data);
     if (LIBBALSA_IS_MAILBOX_IMAP(mbnode->mailbox))
 	/* it already has a mailbox */
@@ -915,10 +889,10 @@ add_imap_mailbox(GNode*root, const char* fn, char delim)
 }
 
 static GNode*
-add_imap_folder(GNode*root, const char* fn, char delim)
+add_imap_folder(GNode*root, const char* fn, char delim, gboolean scanned)
 { 
     if(balsa_app.debug) 
 	printf("add_imap_folder: Adding folder of path %s\n", fn);
-    return add_imap_entry(root, fn, delim);
+    return add_imap_entry(root, fn, delim, scanned);
 }
 
