@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2002 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -258,17 +258,19 @@ threads_destroy(void)
 static gboolean
 initial_open_unread_mailboxes()
 {
-    GList *i, *gl = mblist_find_all_unread_mboxes();
+    GList *i, *gl;
+    
+    gdk_threads_enter();
+    gl = mblist_find_all_unread_mboxes();
 
-    if (!gl)
-	return FALSE;
-
-    for (i = g_list_first(gl); i; i = g_list_next(i)) {
-	printf("opening %s..\n", (LIBBALSA_MAILBOX(i->data))->name);
-	mblist_open_mailbox(LIBBALSA_MAILBOX(i->data));
+    if (gl) {
+        for (i = g_list_first(gl); i; i = g_list_next(i)) {
+            printf("opening %s..\n", (LIBBALSA_MAILBOX(i->data))->name);
+            mblist_open_mailbox(LIBBALSA_MAILBOX(i->data));
+        }
+        g_list_free(gl);
     }
-    g_list_free(gl);
-
+    gdk_threads_leave();
     return FALSE;
 }
 
@@ -282,12 +284,35 @@ initial_open_inbox()
 	return FALSE;
 
     printf("opening %s..\n", (LIBBALSA_MAILBOX(balsa_app.inbox))->name);
+    gdk_threads_enter();
     mblist_open_mailbox(LIBBALSA_MAILBOX(balsa_app.inbox));
+    gdk_threads_leave();
     
     return FALSE;
 }
 
+static gint
+append_subtree_f(GNode* gn, gpointer data)
+{
+    g_return_val_if_fail(gn->data, FALSE);
+    balsa_mailbox_node_append_subtree(BALSA_MAILBOX_NODE(gn->data),
+				      gn);
+    return FALSE;
+}
 
+/* scan_mailboxes:
+   this is an idle handler. Expands subtrees
+*/
+static gboolean
+scan_mailboxes_idle_cb()
+{
+    gdk_threads_enter();
+    g_node_traverse(balsa_app.mailbox_nodes, G_POST_ORDER, G_TRAVERSE_ALL, -1,
+                    append_subtree_f, NULL);
+    balsa_mblist_repopulate(balsa_app.mblist);
+    gdk_threads_leave();
+    return FALSE; 
+}
 /* -------------------------- main --------------------------------- */
 int
 main(int argc, char *argv[])
@@ -298,6 +323,9 @@ main(int argc, char *argv[])
 #ifdef GTKHTML_HAVE_GCONF
     GError *gconf_error;
 #endif
+#ifdef BALSA_USE_THREADS
+    pthread_t scan_thread;
+#endif
 
 #ifdef ENABLE_NLS
     /* Initialize the i18n stuff */
@@ -324,13 +352,8 @@ main(int argc, char *argv[])
     /* Initialize libbalsa */
     libbalsa_init((LibBalsaInformationFunc) balsa_information);
 
-#ifdef USE_PIXBUF
     gtk_widget_set_default_colormap(gdk_rgb_get_cmap());
     gtk_widget_set_default_visual(gdk_rgb_get_visual());
-#else
-    gtk_widget_set_default_colormap(gdk_imlib_get_colormap());
-    gtk_widget_set_default_visual(gdk_imlib_get_visual());
-#endif
 
     /* Allocate the best colormap we can get */
     balsa_app.visual = gdk_visual_get_best();
@@ -338,14 +361,12 @@ main(int argc, char *argv[])
 
     /* checking for valid config files */
     config_init();
-
     /* load mailboxes */
     mailboxes_init();
 
     /* create all the pretty icons that balsa uses that
      * arn't part of gnome-libs */
     balsa_icons_init();
-
     default_icon = balsa_pixmap_finder("balsa/balsa_icon.png");
 #ifdef HAVE_LIBGNOMEUI_GNOME_WINDOW_ICON_H
     gnome_window_icon_set_default_from_file(default_icon);
@@ -378,8 +399,7 @@ main(int argc, char *argv[])
 	    else sendmsg_window_set_field(snd,"to", opt_compose_email);
 	}
 	for(lst = opt_attach_list; lst; lst = g_slist_next(lst))
-	    add_attachment(GNOME_ICON_LIST(snd->attachments[1]), 
-			   lst->data, FALSE, NULL);
+	    add_attachment(snd, lst->data, FALSE, NULL);
 	SENDMSG_WINDOW_QUIT_ON_CLOSE(snd);
     } else
 	gtk_widget_show(window);
@@ -394,12 +414,18 @@ main(int argc, char *argv[])
 	gtk_idle_add((GtkFunction) initial_open_inbox, NULL);
 
     if (cmd_line_open_mailboxes) {
-	gchar **names = g_strsplit(cmd_line_open_mailboxes, ";", 20);
-	gtk_idle_add((GtkFunction) open_mailboxes_idle_cb, names);
+	gchar **urls = g_strsplit(cmd_line_open_mailboxes, ";", 20);
+	gtk_idle_add((GtkFunction) open_mailboxes_idle_cb, urls);
     }
-
     signal( SIGPIPE, SIG_IGN );
 
+#ifdef BALSA_USE_THREADS
+    pthread_create(&scan_thread, NULL, 
+                   (void*(*)(void*))scan_mailboxes_idle_cb, NULL);
+    pthread_detach(scan_thread);
+#else
+    gtk_idle_add((GtkFunction) scan_mailboxes_idle_cb, NULL);
+#endif
     gdk_threads_enter();
     gtk_main();
     gdk_threads_leave();
@@ -459,6 +485,27 @@ balsa_cleanup(void)
 
     config_save();
 
+#ifdef BALSA_USE_THREADS
+    /* move threads shutdown to separate routine?
+       There are actually many things to do, e.g. threads should not
+       be started after this point.
+    */
+    pthread_mutex_lock(&mailbox_lock);
+    if(checking_mail) {
+        /* We want to quit but there is a checking thread active.
+           The alternatives are to:
+           a. wait for the checking thread to finish - but it could be
+           time consuming.  
+           b. send cancel signal to it. 
+        */
+        pthread_cancel(get_mail_thread);
+        libbalsa_unlock_mutt();
+        printf("Mail check thread cancelled. I know it is rough.\n");
+        sleep(1);
+    }
+    pthread_mutex_unlock(&mailbox_lock);
+#endif
+
     /* FIXME: stop switching notebook pages in a more elegant way.
        Probably, the cleanest solution is to call enable_menus_xxx
        functions from an idle function connected to balsa_message_set. */
