@@ -23,11 +23,6 @@
 
 #include <string.h>
 #include <gnome.h>
-#include <gdk/gdkx.h>
-
-#ifdef USE_PIXBUF
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#endif
 
 #ifdef BALSA_USE_THREADS
 #include <pthread.h>
@@ -99,6 +94,7 @@ static char *toolbar0_legal[]={
     BALSA_PIXMAP_SHOW_PREVIEW,
     BALSA_PIXMAP_TRASH,
     BALSA_PIXMAP_TRASH_EMPTY,
+    BALSA_PIXMAP_TOGGLE_FLAGGED,
     NULL
 };
 
@@ -130,12 +126,14 @@ static char *toolbar2_legal[]={
     BALSA_PIXMAP_SHOW_HEADERS,
     BALSA_PIXMAP_TRASH,
     BALSA_PIXMAP_TRASH_EMPTY,
+    BALSA_PIXMAP_TOGGLE_FLAGGED,
     GNOME_STOCK_PIXMAP_CLOSE,
     NULL
 };
 
 static char **toolbar_legal[]={toolbar0_legal, toolbar1_legal, toolbar2_legal};
 
+/* The descriptions must be SHORT */
 button_data toolbar_buttons[]={
     {"", N_("Separator"), "", 0},
     {BALSA_PIXMAP_RECEIVE, N_("Check"),
@@ -168,13 +166,13 @@ button_data toolbar_buttons[]={
      N_("Print current message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_SEND, N_("Send"),
      N_("Send this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_SEND_RECEIVE, N_("Send /\nReceive"),
+    {BALSA_PIXMAP_SEND_RECEIVE, N_("Exchange"),
      N_("Send and Receive messages"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_ATTACHMENT, N_("Attach"),
      N_("Add attachments to this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_SAVE, N_("Save"),
      N_("Save the current item"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_IDENTITY, N_("Select Identity"),
+    {BALSA_PIXMAP_IDENTITY, N_("Identity"),
      N_("Set identity to use for this message"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {GNOME_STOCK_PIXMAP_SPELLCHECK, N_("Spelling"),
      N_("Run a spell check"), TOOLBAR_BUTTON_TYPE_BUTTON},
@@ -190,8 +188,10 @@ button_data toolbar_buttons[]={
      N_("Delete messages from the trash mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
     {BALSA_PIXMAP_CLOSE_MBOX, N_("Close"),
      N_("Close current mailbox"), TOOLBAR_BUTTON_TYPE_BUTTON},
-    {BALSA_PIXMAP_SHOW_PREVIEW, N_("Preview\npane"),
-     N_("Show preview pane"), TOOLBAR_BUTTON_TYPE_TOGGLE}
+    {BALSA_PIXMAP_SHOW_PREVIEW, N_("Msg Preview"),
+     N_("Show preview pane"), TOOLBAR_BUTTON_TYPE_TOGGLE},
+    {BALSA_PIXMAP_TOGGLE_FLAGGED, N_("Toggle\nflagged"),
+     N_("Toggle message is flagged"), TOOLBAR_BUTTON_TYPE_BUTTON}
 };
 
 const int toolbar_button_count = sizeof(toolbar_buttons)
@@ -201,14 +201,14 @@ static void populate_stock_toolbar(int bar, BalsaToolbarType id);
 static int get_toolbar_button_slot(BalsaToolbarType toolbar, const char *id);
 static GtkToolbar *get_bar_instance(GtkWidget *window, 
 				    BalsaToolbarType toolbar);
-static int get_position_value(BalsaToolbarType toolbar, char *id);
+static int get_position_value(BalsaToolbarType toolbar, const char *id);
 
 #ifdef NEW_GTK
-#define mygtk_toolbar_remove_all(bar,j) gtk_toolbar_remove_all(bar)
+#define balsa_toolbar_remove_all(bar,j) gtk_toolbar_remove_all(bar)
 #else
 /* this should go to GTK because it modifies its internal structures. */
-static void
-mygtk_toolbar_remove_all(GtkToolbar *toolbar)
+void
+balsa_toolbar_remove_all(GtkToolbar *toolbar)
 {
     GList *children;
     
@@ -218,10 +218,14 @@ mygtk_toolbar_remove_all(GtkToolbar *toolbar)
 	GtkToolbarChild *child = children->data;
 	
 	if (child->type != GTK_TOOLBAR_CHILD_SPACE) {
+#ifdef REMOVE_TOOLBAR_ITEMS_VERY_CAUTIOUSLY
 	    gtk_widget_ref (child->widget);
 	    gtk_widget_unparent (child->widget);
 	    gtk_widget_destroy (child->widget);
 	    gtk_widget_unref (child->widget);
+#else 
+	    gtk_widget_unparent (child->widget);
+#endif /* REMOVE_TOOLBAR_ITEMS_VERY_CAUTIOUSLY */
 	}
 	g_free (child);
     }
@@ -232,7 +236,7 @@ mygtk_toolbar_remove_all(GtkToolbar *toolbar)
 #endif
 
 static int
-get_position_value(BalsaToolbarType toolbar, char *id)
+get_position_value(BalsaToolbarType toolbar, const char *id)
 {
     int i;
 
@@ -258,10 +262,10 @@ get_position_value(BalsaToolbarType toolbar, char *id)
    		GtkWidget *, or NULL if error / not found
 */
 GtkWidget *
-get_tool_widget(GtkWidget *window, BalsaToolbarType toolbar, char *id)
+get_tool_widget(GtkWidget *window, BalsaToolbarType toolbar, const char *id)
 {
     GtkToolbar *bar;
-    GList *lp, *children;
+    GList *children;
     int position;
     GtkWidget *child;
 
@@ -270,28 +274,13 @@ get_tool_widget(GtkWidget *window, BalsaToolbarType toolbar, char *id)
 	return NULL;
     
     position=get_position_value(toolbar, id);
-    if(position == -1)
+    if (position < 0)
 	return NULL;
     
-    lp=children=gtk_container_children(GTK_CONTAINER(bar));
-    if(!children)
-	return(NULL);
+    children = gtk_container_children(GTK_CONTAINER(bar));
+    child = GTK_WIDGET(g_list_nth_data(children, position));
+    g_list_free(children);
 
-    lp = g_list_nth(lp, position);
-
-    if(!lp) {
-	g_list_free(children);
-	return NULL;
-    }
-	
-    child=(GtkWidget *)(lp->data);
-
-    if(children)
-	g_list_free(children);
-	
-    if(!child)
-	return NULL;
-	
     return child;
 }
 
@@ -518,7 +507,7 @@ get_toolbar(GtkWidget *window, BalsaToolbarType toolbar)
     } else {
 	bar=GTK_TOOLBAR(toolbar_map[i].toolbar);
 	/* remove all items from the existing bar. */
-	mygtk_toolbar_remove_all(bar);
+	balsa_toolbar_remove_all(bar);
     }
 
     toolbar_map[i].toolbar=GTK_WIDGET(bar);
@@ -671,7 +660,7 @@ set_toolbar_button_callback(BalsaToolbarType toolbar, const char *id,
 */
 void
 set_toolbar_button_sensitive(GtkWidget *window, BalsaToolbarType toolbar, 
-			     char *id, int sensitive)
+			     const char *id, int sensitive)
 {
     int slot;
     GtkWidget *widget;
