@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2002 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,6 @@
 
 #include <string.h>
 #include <gnome.h>
-#include <gdk/gdkx.h>
-
-#ifdef USE_PIXBUF
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#endif
 
 #ifdef BALSA_USE_THREADS
 #include <pthread.h>
@@ -113,7 +108,6 @@ static void get_toolbar_data(BalsaToolbarType toolbar);
 static void apply_toolbar_prefs(GtkWidget *widget, gpointer data);
 static void wrap_toggled_cb(GtkWidget *widget, gpointer data);
 static void page_active_cb(GtkWidget *widget, GdkEvent *event, gpointer data);
-static void remove_all_buttons(GtkToolbar *bar);
 
 static void
 page_active_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
@@ -251,27 +245,6 @@ get_toolbar_data(BalsaToolbarType toolbar)
     }
 }
 
-static void
-remove_all_buttons(GtkToolbar *bar)
-{
-    GList *lp;
-    GtkToolbarChild *child;
-    
-    while((lp=g_list_first(bar->children)) != NULL) {
-	bar->children=g_list_remove_link(bar->children, lp);
-	
-	child=(GtkToolbarChild *)lp->data;
-	if(child->label)
-	    gtk_widget_destroy(GTK_WIDGET(child->label));
-	if(child->icon)
-	    gtk_widget_destroy(GTK_WIDGET(child->icon));
-	if(child->widget)
-	    gtk_widget_destroy(GTK_WIDGET(child->widget));
-	g_free(child);
-	g_list_free(lp);
-    }
-}
-
 static void
 replace_nl_with_space(char* str)
 {
@@ -296,7 +269,7 @@ recreate_preview(BalsaToolbarType toolbar, gboolean preview_only)
     list_data[0] = list_data[1] = NULL;
     bar=GTK_TOOLBAR(toolbar_pages[toolbar].preview);
     
-    remove_all_buttons(bar);
+    balsa_toolbar_remove_all(bar);
     if(!preview_only) {
 	gtk_clist_clear(
 	    GTK_CLIST(toolbar_pages[toolbar].destination));
@@ -393,6 +366,10 @@ back_button_cb(GtkWidget *widget, gpointer data)
     gtk_clist_moveto(GTK_CLIST(toolbar_pages[toolbar].destination),
 		     row-1, 0, 0, 0);
     gnome_property_box_changed(GNOME_PROPERTY_BOX(customize_widget));
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].back_button, 
+			     toolbar_pages[toolbar].selected_destination > 0);
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].forward_button, 
+			     TRUE);
 }
 
 static void
@@ -424,6 +401,11 @@ forward_button_cb(GtkWidget *widget, gpointer data)
     gtk_clist_moveto(GTK_CLIST(toolbar_pages[toolbar].destination),
 		     row-1, 0, 0, 0);
     gnome_property_box_changed(GNOME_PROPERTY_BOX(customize_widget));
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].back_button, 
+			     TRUE);
+    gtk_widget_set_sensitive(toolbar_pages[toolbar].forward_button, 
+			     toolbar_pages[toolbar].selected_destination <
+			     toolbar_item_count[toolbar]-1);
 }
 
 static void
